package pwc.taxtech.atms.vat.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.util.DataUtil;
import pwc.taxtech.atms.common.util.FileExcelUtil;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.*;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.PeriodReportDto;
import pwc.taxtech.atms.dto.ReportAttachDto;
import pwc.taxtech.atms.dto.RequestParameterDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.NotFoundException;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.service.impl.*;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.DataSourceCellDataDto;
import pwc.taxtech.atms.vat.dpo.DataSourceExtendDto;
import pwc.taxtech.atms.vat.dpo.InputVATInvoiceItemExtendDto;
import pwc.taxtech.atms.vat.dpo.PeriodCellTemplateConfigDto;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaHelper;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.dto.vatdto.WrapPeriodJobDto.*;

@Component
public class ReportServiceImpl extends BaseService {
    private final static Logger logger = LoggerFactory.getLogger(ReportServiceImpl.class);
    private BlockingQueue<PeriodJob> queue = new LinkedBlockingQueue<>();
    private final static String[] functions = {"SGSR", "FSJZ", "ND", "BB", "XXFP", "GZSD", "PC", "JXFPMX",
            "JXFP", "PSUM", "DFFS", "JFFS", "WPSR", "WPNAME", "WPTYPE", "SUM2", "RSUMIF", "SUM", "KPSR", "TBM"};

    @Autowired
    private ReportGeneratorImpl reportGenerator;
    @Resource
    private PeriodTaxRuleSettingMapper periodTaxRuleSettingMapper;
    @Resource
    private PeriodTaxPayerReportRuleMapper periodTaxPayerReportRuleMapper;
    @Resource
    private PeriodEnterpriseAccountMapper periodEnterpriseAccountMapper;
    @Autowired
    private InputVatInvoiceMapper inputVATInvoiceMapper;
    @Autowired
    private OutputVatInvoiceMapper outputVATInvoiceMapper;
    @Resource
    private CustomsInvoiceMapper customsInvoiceMapper;
    @Resource
    private PeriodCellDataSourceMapper periodCellDataSourceMapper;
    @Resource
    private KeyValueConfigMapper keyValueConfigMapper;
    @Resource
    private PeriodModifiedReportCellMapper periodModifiedReportCellMapper;
    @Resource
    private PeriodReportMapper periodReportMapper;
    @Resource
    private EnterpriseAccountMapper enterpriseAccountMapper;
    @Resource
    private CellTemplateConfigMapper cellTemplateConfigMapper;
    @Resource
    private CellTemplateMapper cellTemplateMapper;
    @Resource
    private PeriodDataSourceDetailMapper periodDataSourceDetailMapper;
    @Resource
    private PeriodDataSourceMapper periodDataSourceMapper;
    @Resource
    private PeriodCellDataMapper periodCellDataMapper;
    @Resource
    private PeriodFormulaBlockMapper periodFormulaBlockMapper;
    @Resource
    private PeriodCellTemplateConfigMapper periodCellTemplateConfigMapper;
    @Resource
    private PeriodCellTemplateMapper periodCellTemplateMapper;
    @Resource
    private PeriodTemplateMapper periodTemplateMapper;
    @Resource
    private TemplateMapper templateMapper;
    @Resource
    private ProjectServiceTypeMapper projectServiceTypeMapper;
    @Resource
    private TaxPayerReportRuleMapper taxPayerReportRuleMapper;
    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private DistributedIdService distributedIdService;
    @Resource
    private PeriodJobMapper periodJobMapper;
    @Resource
    private PeriodApproveMapper periodApprovalMapper;
    @Resource
    RevenueConfigMapper revenueConfigMapper;
    @Resource
    RevenueTypeMappingMapper revenueTypeMappingMapper;
    @Resource
    InvoiceRecordMapper invoiceRecordMapper;
    @Autowired
    private HttpFileService httpFileService;
    @Autowired
    DidiFileUploadService didiFileUploadService;
    @Resource
    TrialBalanceFinalMapper trialBalanceFinalMapper;
    @Autowired
    FormulaAgent agent;

    public OperationResultDto<List<ReportDto>> getFilterReportTemplate(String projectId, EnumServiceType serviceType, Integer periodParam) {
        OperationResultDto<List<ReportDto>> result = new OperationResultDto<>();
        CommonUtils.copyProperties(getReportTemplate(projectId, serviceType, periodParam), result);
        if (!result.getResult() || CollectionUtils.isEmpty(result.getData())) return result;
        int period = periodParam != null ? periodParam : 0;
        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId)
                .andPeriodEqualTo(period);
        List<PeriodTemplate> periodTemplateList = periodTemplateMapper.selectByExample(periodTemplateExample);
        List<String> ids = Lists.newArrayList();
        periodTemplateList.stream().forEach(x -> ids.add(x.getTemplateId() + ""));

        List<ReportDto> reportDtos = result.getData().stream().filter(x -> {
            return x.getId() != null && ids.contains(x.getTemplateId());
        }).collect(Collectors.toList());
        result.setData(reportDtos);
        return result;
    }

    public OperationResultDto<List<ReportDto>> getReportTemplate(String projectId, EnumServiceType serviceType, Integer periodParam) {
        int period = periodParam != null ? periodParam : 0;
        OperationResultDto<List<ReportDto>> operationResult = new OperationResultDto<>();
        try {
            Project project = projectMapper.selectByPrimaryKey(projectId);
            String organizationId = project == null ? null : project.getOrganizationId();
            if (organizationId == null) {
                operationResult.setResult(false);
                operationResult.setResultMsg("NoOrganization");
                return operationResult;
            }
            Long templateGroupId;

        /*
        对于VAT，从Rule Engine中获取TemplateGroupId
        对于其他Service Type，直接从ProjectServiceType中获取
         */
            if (serviceType.equals(EnumServiceType.VAT)) {
                TaxPayerReportRuleExample example = new TaxPayerReportRuleExample();
                example.createCriteria().andOrganizationIdLike(organizationId);
                List<TaxPayerReportRule> taxPayerReportRules = taxPayerReportRuleMapper.selectByExample(example);
                Optional<TaxPayerReportRule> taxPayerReportRule = taxPayerReportRules.stream().findFirst();
                if (!taxPayerReportRule.isPresent()) {
                    Optional<TaxPayerReportRule> defaultRule = taxPayerReportRules.stream().filter(x -> x.getIsDefault()).findFirst();
                    if (!defaultRule.isPresent()) {
                        operationResult.setResultMsg("NoDefaultRule");
                        return operationResult;
                    }

                    templateGroupId = defaultRule.get().getTemplateGroupId();
                } else {
                    templateGroupId = taxPayerReportRule.get().getTemplateGroupId();
                }

            } else {
                String serviceTypeStr = serviceType.getCode().toString();
                ProjectServiceTypeExample projectServiceTypeExample = new ProjectServiceTypeExample();
                projectServiceTypeExample.createCriteria().andServiceTypeIdEqualTo(serviceTypeStr).andProjectIdEqualTo(projectId);
                Optional<Long> tempVlaue = projectServiceTypeMapper.selectByExample(projectServiceTypeExample).stream().map(ProjectServiceType::getTemplateGroupId).findFirst();
                if (tempVlaue.isPresent()) {
                    templateGroupId = tempVlaue.get();
                } else {
                    templateGroupId = 0L;
                }
            }

            Map<String, Object> map = new HashMap<>();
            map.put("period", period);
            map.put("projectId", projectId);
            map.put("templateGroupID", templateGroupId);


            List<ReportDto> reportDtos = templateMapper.getTemplateLeftJoinReport(map);


            if (reportDtos.isEmpty()) {
                operationResult.setResultMsg("No Template");
                return operationResult;
            }

            operationResult.setResult(true);
            operationResult.setData(reportDtos);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResult.setResult(false);
        }
        return operationResult;
    }

    private void clearPeriodData(String projectId, Integer period, List<Long> templateIds, boolean isMergeManualData) {
        if (templateIds.isEmpty()) templateIds.add(Long.MAX_VALUE);
        PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
        periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodFormulaBlockMapper.deleteByExample(periodFormulaBlockExample);

        PeriodTaxRuleSettingExample periodTaxRuleSettingExample = new PeriodTaxRuleSettingExample();
        periodTaxRuleSettingExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxRuleSettingMapper.deleteByExample(periodTaxRuleSettingExample);

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateMapper.deleteByExample(periodCellTemplateExample);

        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        periodCellTemplateConfigExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateConfigMapper.deleteByExample(periodCellTemplateConfigExample);

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andTemplateIdNotIn(templateIds);
        periodTemplateMapper.deleteByExample(periodTemplateExample);

        PeriodTaxPayerReportRuleExample periodTaxPayerReportRuleExample = new PeriodTaxPayerReportRuleExample();
        periodTaxPayerReportRuleExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxPayerReportRuleMapper.deleteByExample(periodTaxPayerReportRuleExample);

        PeriodDataSourceExample periodDataSourceExample = new PeriodDataSourceExample();
        periodDataSourceExample.createCriteria().andProjectIdEqualTo(projectId)
                .andPeriodEqualTo(period);
        List<PeriodDataSource> periodDataSources = periodDataSourceMapper.selectByExample(periodDataSourceExample);

        Set<Long> periodDsIds = Sets.newHashSet(Long.MAX_VALUE);
        List<Long> periodManualDsIds = new ArrayList<>();

        if (isMergeManualData) {
            periodDataSources.forEach(m -> {
                if (m.getType().intValue() != FormulaDataSourceType.KeyInSource.getCode().intValue()) {
                    periodDsIds.add(m.getId());
                } else {
                    periodManualDsIds.add(m.getId());
                }
            });

        } else {
            periodDsIds.addAll(periodDataSources.stream().map(m -> m.getId()).collect(Collectors.toList()));
        }

        PeriodDataSourceDetailExample periodDataSourceDetailExample = new PeriodDataSourceDetailExample();
        periodDataSourceDetailExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andDataSourceIdIn(Lists.newArrayList(periodDsIds));
        periodDataSourceDetailMapper.deleteByExample(periodDataSourceDetailExample);

        PeriodCellDataSourceExample periodCellDataSourceExample = new PeriodCellDataSourceExample();
        periodCellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodCellDataSourceMapper.deleteByExample(periodCellDataSourceExample);


        PeriodDataSourceExample periodDataSourceDeleteExample = new PeriodDataSourceExample();
        periodDataSourceDeleteExample.createCriteria().andIdIn(Lists.newArrayList(periodDsIds));
        periodDataSourceMapper.deleteByExample(periodDataSourceDeleteExample);

        PeriodReportExample periodReportExample = new PeriodReportExample();
        periodReportExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodReportMapper.deleteByExample(periodReportExample);

        PeriodCellDataExample periodCellDataExample = new PeriodCellDataExample();
        periodCellDataExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodCellDataMapper.deleteByExample(periodCellDataExample);


    }

    private void updateConfig(String projectId, Integer period, Boolean isMergeManualData, List<Template> templates, PeriodJob job) {
        List<Long> exceptTemplateIds = templateMapper.getIdsForExceptTemplate();
        //根据收入类型映射生成开票记录关联数据
        assembleInvoiceRecord(projectId, period, isMergeManualData);
        //生成trial_balance_final数据
        assembleTrialBalanceFinal(projectId, period);
        clearPeriodData(projectId, period, exceptTemplateIds, isMergeManualData);
        copyTemplateAndConfigFromAdmin(projectId, templates, period);
        setStatus(job, STATUS_END);
        periodJobMapper.updateByPrimaryKey(job);
    }

    public void assembleTrialBalanceFinal(String projectId, Integer period) {
        Project project = projectMapper.selectByPrimaryKey(projectId);
        MyAsserts.assertNotNull(project, Exceptions.NOT_FOUND_REPORT_EXCEPTION);
        String queryPeriod = project.getYear() + "" + (period < 10 ? ("0" + period) : (period + ""));
        Project lastProject = agent.getFixedProject(projectId, project.getYear());
        String lastPeriod = "0";
        if (lastProject != null) {
            if (project.getId().equals(lastProject.getId())) {
                lastPeriod = project.getYear() + "" + (period - 1 < 10 ? ("0" + (period - 1)) : ((period - 1) + ""));
            } else {
                lastPeriod = lastProject.getYear() + "12";
            }
        }
        TrialBalanceFinalExample example = new TrialBalanceFinalExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(Integer.valueOf(queryPeriod));
        trialBalanceFinalMapper.deleteByExample(example);
        trialBalanceFinalMapper.generateFinalData(projectId, Integer.valueOf(queryPeriod), lastProject == null ? "0" : lastProject.getId(), Integer.valueOf(lastPeriod));
    }

    public void assembleInvoiceRecord(String projectId, Integer period, Boolean isMergeManualData) {
        Project project = projectMapper.selectByPrimaryKey(projectId);
        MyAsserts.assertNotNull(project, Exceptions.NOT_FOUND_REPORT_EXCEPTION);
        String queryDate = project.getYear() + "-" + (period < 10 ? ("0" + period) : (period + ""));
        RevenueTypeMappingExample mappingExample = new RevenueTypeMappingExample();
        mappingExample.createCriteria().andOrgIdEqualTo(project.getOrganizationId()).
                andStartDateLessThanOrEqualTo(queryDate).andEndDateGreaterThanOrEqualTo(queryDate).
                andStatusEqualTo(0);
        List<RevenueTypeMapping> mappingList = revenueTypeMappingMapper.selectByExample(mappingExample);
        //先清除数据
        InvoiceRecord delRecord = new InvoiceRecord();
        delRecord.setRevenueCofId(null);
        if (!isMergeManualData) {
            delRecord.setModifyRevenueCofId(null);
        }
        InvoiceRecordExample delExample = new InvoiceRecordExample();
        delExample.createCriteria().andProjectIdEqualTo(projectId)
                .andProjectIdEqualTo(projectId).
                andPeriodEqualTo(Integer.valueOf(queryDate.replace("-", "")));
        invoiceRecordMapper.deleteByExample(delExample);
        Map<String, Long> map = new HashMap<>();
        for (RevenueTypeMapping mapping : mappingList) {
            if (!map.containsKey(mapping.getContent())) {
                RevenueConfigExample configExample = new RevenueConfigExample();
                configExample.createCriteria().andNameEqualTo(mapping.getRevenueTypeName()).
                        andStatusEqualTo(0).andOrgIdEqualTo(mapping.getOrgId()).
                        andStartDateLessThanOrEqualTo(queryDate).andEndDateGreaterThanOrEqualTo(queryDate);
                List<RevenueConfig> configs = revenueConfigMapper.selectByExample(configExample);
                if (CollectionUtils.isNotEmpty(configs)) {
                    map.put(mapping.getContent(), configs.get(0).getId());
                }
            }
            if (map.containsKey(mapping.getContent())) {
                InvoiceRecord destRecord = new InvoiceRecord();
                destRecord.setRevenueCofId(map.get(mapping.getContent()));
                InvoiceRecordExample recordExample = new InvoiceRecordExample();
                recordExample.createCriteria().andProjectIdEqualTo(projectId)
                        .andOrganizationIdEqualTo(mapping.getOrgId()).andBillingContentEqualTo(mapping.getContent()).
                        andPeriodEqualTo(Integer.valueOf(queryDate.replace("-", "")));
                invoiceRecordMapper.updateByExampleSelective(destRecord, recordExample);
            }
        }
    }

    private List<Template> getTemplatesByProjectId(String projectId) {
        MyAsserts.assertNotEmpty(projectId, Exceptions.PROJECT_PROJECT_EXCEPTION);
        Project project = projectMapper.selectByPrimaryKey(projectId);
        MyAsserts.assertNotNull(project, Exceptions.NOT_FOUND_REPORT_EXCEPTION);

        Long templateGroupId = projectMapper.getTemplateGroupIdByProject(projectId, EnumServiceType.VAT.getCode());

        MyAsserts.assertNotNull(templateGroupId, Exceptions.NOT_FOUND_TEMPLATE_GROUP_EXCEPTION);
        List<Template> templates = queryTemplateByGroup(templateGroupId);
        MyAsserts.assertNotEmpty(templates, Exceptions.NOT_FOUND_TEMPLATE_EXCEPTION);

        templates.sort(Comparator.comparing(Template::getOrderIndex));
        return templates;
    }

    private void copyPeriodConfigFromCellTemplateConfig(String projectId, Long templateId, Integer period) {
        CellTemplateConfigExample cellTemplateConfigExample = new CellTemplateConfigExample();
        cellTemplateConfigExample.createCriteria().andReportTemplateIdEqualTo(templateId);
        List<CellTemplateConfig> cellTemplateConfigList = cellTemplateConfigMapper.selectByExample(cellTemplateConfigExample);
        if (cellTemplateConfigList.isEmpty()) return;

        List<PeriodCellTemplateConfig> periodCellTemplateConfigList = new ArrayList<>();


        for (CellTemplateConfig cellTemplateConfig : cellTemplateConfigList) {
            PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
            CommonUtils.copyProperties(cellTemplateConfig, periodCellTemplateConfig);

            periodCellTemplateConfig.setId(distributedIdService.nextId());
            periodCellTemplateConfig.setPeriod(period);
            periodCellTemplateConfig.setCellTemplateId(cellTemplateConfig.getCellTemplateId());
            periodCellTemplateConfig.setReportTemplateId(templateId);
            periodCellTemplateConfig.setDataSourceType(cellTemplateConfig.getDataSourceType());
            periodCellTemplateConfig.setFormula(StringUtils.isBlank(cellTemplateConfig.getFormula()) ? null : cellTemplateConfig.getFormula());
            periodCellTemplateConfig.setParsedFormula(null);
            periodCellTemplateConfig.setFormulaDescription(cellTemplateConfig.getFormulaDescription());
            periodCellTemplateConfig.setAccountCodes(cellTemplateConfig.getAccountCodes());
            periodCellTemplateConfig.setInvoiceType(cellTemplateConfig.getInvoiceType());
            periodCellTemplateConfig.setTaxRate(cellTemplateConfig.getTaxRate());
            periodCellTemplateConfig.setInvoiceAmountType(cellTemplateConfig.getInvoiceAmountType());
            periodCellTemplateConfig.setModelIds(cellTemplateConfig.getModelIds());
            periodCellTemplateConfig.setCreateBy(StringUtils.isBlank(cellTemplateConfig.getCreateBy()) ? "admin" : cellTemplateConfig.getCreateBy());
            periodCellTemplateConfig.setCreateTime(cellTemplateConfig.getCreateTime());
            periodCellTemplateConfig.setUpdateBy(StringUtils.isBlank(cellTemplateConfig.getUpdateBy()) ? "admin" : cellTemplateConfig.getUpdateBy());
            periodCellTemplateConfig.setUpdateTime(cellTemplateConfig.getUpdateTime());
            periodCellTemplateConfig.setInvoiceCategory(cellTemplateConfig.getInvoiceCategory());
            periodCellTemplateConfig.setFormulaDataSource(cellTemplateConfig.getFormulaDataSource());
            periodCellTemplateConfig.setValidation(cellTemplateConfig.getValidation());
            periodCellTemplateConfig.setParsedValidation(null);
            periodCellTemplateConfig.setValidationDescription(cellTemplateConfig.getValidationDescription());
            periodCellTemplateConfig.setVoucherKeyword(cellTemplateConfig.getVoucherKeyword());
            periodCellTemplateConfig.setCellTemplateConfigId(cellTemplateConfig.getId());
            periodCellTemplateConfig.setKeyValueParsedFormula(null);
            periodCellTemplateConfig.setProjectId(projectId);

            fixedParsedFormula(periodCellTemplateConfig);
            fixedAccountCode(periodCellTemplateConfig);

            periodCellTemplateConfigList.add(periodCellTemplateConfig);
        }

        CommonUtils.subListWithLen(periodCellTemplateConfigList, CommonUtils.BATCH_NUM).forEach(batch -> {
            periodCellTemplateConfigMapper.batchInsert(batch);
        });

    }

    @Transactional
    public void assemblePeriodTemplate(Template template, Workbook workbook, String projectId, Integer period, List<Long> configIds) throws ServiceException {
        int addRowIndex = configIds.size();
        List<Integer> hasHandDatas = Arrays.asList(
                TaxesCalculateReportEnum.Column.Column_4.getIndex(),
                TaxesCalculateReportEnum.Column.Column_5.getIndex(),
                TaxesCalculateReportEnum.Column.Column_6.getIndex(),
                TaxesCalculateReportEnum.Column.Column_7.getIndex(),
                TaxesCalculateReportEnum.Column.Column_8.getIndex(),
                TaxesCalculateReportEnum.Column.Column_9.getIndex(),
                TaxesCalculateReportEnum.Column.Column_11.getIndex());
        List<Integer> hasFormulaDatas = Arrays.asList(
                TaxesCalculateReportEnum.Column.Column_3.getIndex(),
                TaxesCalculateReportEnum.Column.Column_4.getIndex(),
                TaxesCalculateReportEnum.Column.Column_5.getIndex(),
                TaxesCalculateReportEnum.Column.Column_6.getIndex(),
                TaxesCalculateReportEnum.Column.Column_7.getIndex(),
                TaxesCalculateReportEnum.Column.Column_8.getIndex(),
                TaxesCalculateReportEnum.Column.Column_9.getIndex(),
                TaxesCalculateReportEnum.Column.Column_11.getIndex());
        try {
            Date now = new Date();
            Sheet sheet = workbook.getSheetAt(0);
            List<PeriodCellTemplate> cellTemplateList = Lists.newArrayList();
            List<PeriodCellTemplateConfig> cellTemplateConfigList = Lists.newArrayList();
            for (int r = sheet.getFirstRowNum(); r <= sheet.getLastRowNum(); r++) {
                Row row = sheet.getRow(r);
                for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {

                    Cell cell = row.getCell(c);
                    if (cell == null) {
                        continue;//todo cell == null 如何处理
                    }
                    if (r <= addRowIndex + 1) {
                        String cellId = projectId + template.getId() + period + r + c;
                        if ((r - 1) >= 0 && (r - 1) < configIds.size()) {
                            cellId += configIds.get(configIds.size() - r);
                        }
                        Long cellTemplateId = Long.valueOf(cellId.hashCode());
                        cellTemplateId = cellTemplateId < 0 ? cellTemplateId : (cellTemplateId * -1);
                        //更新手工数据源
                        PeriodDataSourceExample delExample = new PeriodDataSourceExample();
                        delExample.createCriteria().andProjectIdEqualTo(projectId)
                                .andPeriodEqualTo(period).andTypeEqualTo(10).andColumnIndexEqualTo(c).andRowIndexEqualTo(r);
                        PeriodDataSource dataSource = new DataSourceExtendDto();
                        dataSource.setColumnIndex(999);
                        dataSource.setRowIndex(999);
                        periodDataSourceMapper.updateByExampleSelective(dataSource, delExample);

                        PeriodDataSourceExample updateExample = new PeriodDataSourceExample();
                        updateExample.createCriteria().andProjectIdEqualTo(projectId)
                                .andPeriodEqualTo(period).andTypeEqualTo(10).andCellTemplateIdEqualTo(cellTemplateId);
                        dataSource = new DataSourceExtendDto();
                        dataSource.setColumnIndex(c);
                        dataSource.setRowIndex(r);
                        periodDataSourceMapper.updateByExampleSelective(dataSource, updateExample);
                        PeriodCellTemplate cellTemplate = new PeriodCellTemplate();
                        cellTemplate.setPeriod(period);
                        cellTemplate.setRowName(POIUtil.getCellFormulaString(cell));
                        cellTemplate.setProjectId(projectId);
                        cellTemplate.setColumnIndex(c);
                        cellTemplate.setCreateTime(now);
                        cellTemplate.setCellTemplateId(cellTemplateId);
                        cellTemplate.setUpdateTime(now);
                        cellTemplate.setRowIndex(r);
                        cellTemplate.setReportTemplateId(template.getId());
                        cellTemplate.setId(distributedIdService.nextId());
                        if (cell.getCellComment() != null) {
                            cellTemplate.setComment(cell.getCellComment().getString().getString());
                        }
                        cellTemplate.setIsReadOnly(hasHandDatas.contains(c) ? 0 : 1);
                        cellTemplateList.add(cellTemplate);
                        if (r > 0 && hasFormulaDatas.contains(c) && StringUtils.isNotBlank(POIUtil.getCellFormulaString(cell))) {
                            PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
                            periodCellTemplateConfig.setId(distributedIdService.nextId());
                            periodCellTemplateConfig.setPeriod(period);
                            periodCellTemplateConfig.setCellTemplateId(cellTemplateId);
                            periodCellTemplateConfig.setReportTemplateId(template.getId());
                            periodCellTemplateConfig.setDataSourceType(1);
                            periodCellTemplateConfig.setFormula(POIUtil.getCellFormulaString(cell));
                            periodCellTemplateConfig.setParsedFormula(null);
//                            periodCellTemplateConfig.setCreateBy(authUserHelper.getCurrentUserId());
                            periodCellTemplateConfig.setCreateTime(now);
//                            periodCellTemplateConfig.setUpdateBy(authUserHelper.getCurrentUserId());
                            periodCellTemplateConfig.setUpdateTime(now);
                            periodCellTemplateConfig.setFormulaDataSource("报表数据");
                            periodCellTemplateConfig.setKeyValueParsedFormula(null);
                            periodCellTemplateConfig.setProjectId(projectId);
                            fixedParsedFormula(periodCellTemplateConfig);
                            fixedAccountCode(periodCellTemplateConfig);
                            cellTemplateConfigList.add(periodCellTemplateConfig);
                        }
                        if (r > 0 && r <= addRowIndex && hasHandDatas.contains(c)) {
                            addManualConfig(cellTemplate, cell, now, cellTemplateConfigList);
                        }
                    } else {
                        //查询原来行cell是否有CellTemplate
                        CellTemplateExample cellTemplateExample = new CellTemplateExample();
                        cellTemplateExample.createCriteria().andReportTemplateIdEqualTo(template.getId()).andRowIndexEqualTo(r - addRowIndex).andColumnIndexEqualTo(c);
                        List<CellTemplate> cellTemplates = cellTemplateMapper.selectByExample(cellTemplateExample);
                        if (CollectionUtils.isNotEmpty(cellTemplates)) {
                            CellTemplate cellTemplate = cellTemplates.get(0);
                            PeriodCellTemplate periodCellTemplate = new PeriodCellTemplate();
                            CommonUtils.copyProperties(cellTemplate, periodCellTemplate);
                            periodCellTemplate.setId(distributedIdService.nextId());
                            periodCellTemplate.setPeriod(period);
                            periodCellTemplate.setReportTemplateId(template.getId());
                            periodCellTemplate.setRowIndex(r);
                            periodCellTemplate.setRowName(cellTemplate.getRowName());
                            periodCellTemplate.setColumnIndex(c);
                            periodCellTemplate.setColumnName(cellTemplate.getColumnName());
                            periodCellTemplate.setComment(cellTemplate.getComment());
                            periodCellTemplate.setCreateTime(cellTemplate.getCreateTime());
                            periodCellTemplate.setUpdateTime(cellTemplate.getUpdateTime());
                            periodCellTemplate.setCellTemplateId(cellTemplate.getId());
                            periodCellTemplate.setDataType(cellTemplate.getDataType());
                            periodCellTemplate.setIsReadOnly(cellTemplate.getIsReadOnly() ? 1 : 0);
                            periodCellTemplate.setCopyFromId(cellTemplate.getCopyFromId());
                            periodCellTemplate.setCreateBy(StringUtils.isBlank(cellTemplate.getCreateBy()) ? "admin" : cellTemplate.getCreateBy());
                            periodCellTemplate.setUpdateBy(StringUtils.isBlank(cellTemplate.getUpdateBy()) ? "admin" : cellTemplate.getUpdateBy());
                            periodCellTemplate.setProjectId(projectId);
                            cellTemplateList.add(periodCellTemplate);

                            CellTemplateConfigExample configExample = new CellTemplateConfigExample();
                            configExample.createCriteria().andCellTemplateIdEqualTo(cellTemplate.getId());
                            List<CellTemplateConfig> configs = cellTemplateConfigMapper.selectByExample(configExample);
                            if (CollectionUtils.isNotEmpty(configs)) {
                                for (CellTemplateConfig cellTemplateConfig : configs) {
                                    PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
                                    CommonUtils.copyProperties(cellTemplateConfig, periodCellTemplateConfig);
                                    periodCellTemplateConfig.setId(distributedIdService.nextId());
                                    periodCellTemplateConfig.setPeriod(period);
                                    periodCellTemplateConfig.setCellTemplateId(cellTemplateConfig.getCellTemplateId());
                                    periodCellTemplateConfig.setReportTemplateId(template.getId());
                                    periodCellTemplateConfig.setDataSourceType(cellTemplateConfig.getDataSourceType());
                                    periodCellTemplateConfig.setFormula(StringUtils.isBlank(cellTemplateConfig.getFormula()) ? null : cellTemplateConfig.getFormula());
                                    periodCellTemplateConfig.setParsedFormula(null);
                                    periodCellTemplateConfig.setFormulaDescription(cellTemplateConfig.getFormulaDescription());
                                    periodCellTemplateConfig.setAccountCodes(cellTemplateConfig.getAccountCodes());
                                    periodCellTemplateConfig.setInvoiceType(cellTemplateConfig.getInvoiceType());
                                    periodCellTemplateConfig.setTaxRate(cellTemplateConfig.getTaxRate());
                                    periodCellTemplateConfig.setInvoiceAmountType(cellTemplateConfig.getInvoiceAmountType());
                                    periodCellTemplateConfig.setModelIds(cellTemplateConfig.getModelIds());
                                    periodCellTemplateConfig.setCreateBy(StringUtils.isBlank(cellTemplateConfig.getCreateBy()) ? "admin" : cellTemplateConfig.getCreateBy());
                                    periodCellTemplateConfig.setCreateTime(cellTemplateConfig.getCreateTime());
                                    periodCellTemplateConfig.setUpdateBy(StringUtils.isBlank(cellTemplateConfig.getUpdateBy()) ? "admin" : cellTemplateConfig.getUpdateBy());
                                    periodCellTemplateConfig.setUpdateTime(cellTemplateConfig.getUpdateTime());
                                    periodCellTemplateConfig.setInvoiceCategory(cellTemplateConfig.getInvoiceCategory());
                                    periodCellTemplateConfig.setFormulaDataSource(cellTemplateConfig.getFormulaDataSource());
                                    periodCellTemplateConfig.setValidation(cellTemplateConfig.getValidation());
                                    periodCellTemplateConfig.setParsedValidation(null);
                                    periodCellTemplateConfig.setValidationDescription(cellTemplateConfig.getValidationDescription());
                                    periodCellTemplateConfig.setVoucherKeyword(cellTemplateConfig.getVoucherKeyword());
                                    periodCellTemplateConfig.setCellTemplateConfigId(cellTemplateConfig.getId());
                                    periodCellTemplateConfig.setKeyValueParsedFormula(null);
                                    periodCellTemplateConfig.setProjectId(projectId);
                                    fixedParsedFormula(periodCellTemplateConfig);
                                    fixedAccountCode(periodCellTemplateConfig);
                                    cellTemplateConfigList.add(periodCellTemplateConfig);
                                }
                            }
                        }
                    }
                }
            }
            List<List<PeriodCellTemplate>> tmpList = CommonUtils.subListWithLen(cellTemplateList, CommonUtils.BATCH_NUM);
//                tmpList.forEach(list -> cellTemplateMapper.batchInsert2(list));
            tmpList.forEach(list -> periodCellTemplateMapper.batchInsert(list));//todo 批量插入优化
            List<List<PeriodCellTemplateConfig>> tmpConfigList = CommonUtils.subListWithLen(cellTemplateConfigList, CommonUtils.BATCH_NUM);
            tmpConfigList.forEach(list -> periodCellTemplateConfigMapper.batchInsert(list));
        } catch (Exception e) {
            logger.error("importTemplateExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void addManualConfig(PeriodCellTemplate cellTemplate, Cell cell, Date now, List<PeriodCellTemplateConfig> list) {
        PeriodCellTemplateConfig configManual = new PeriodCellTemplateConfig();
        configManual.setId(distributedIdService.nextId());
        configManual.setCellTemplateId(cellTemplate.getCellTemplateId());
        configManual.setReportTemplateId(cellTemplate.getReportTemplateId());
        configManual.setPeriod(cellTemplate.getPeriod());
        configManual.setProjectId(cellTemplate.getProjectId());
        configManual.setDataSourceType(CellDataSourceType.KeyIn.getCode());
        configManual.setFormula(POIUtil.getCellFormulaString(cell));
//        configManual.setFormula(cell.getCellFormula());
        configManual.setFormulaDataSource("报表数据"); //todo KV相关
        configManual.setUpdateTime(now);
//        configManual.setUpdateBy(authUserHelper.getCurrentUserId());
        list.add(configManual);
    }

    /**
     * 替换占位符
     *
     * @param cell cell
     * @return Boolean
     */
    private Boolean hasKeyIn(Cell cell) {
        if (null == cell) {
            return false;
        }
        CellType cellType = cell.getCellTypeEnum();
        if (!CellType.STRING.equals(cellType)) {
            return false;
        }
        String v = cell.getStringCellValue();
        if (StringUtils.isBlank(v)) {
            return false;
        }
        return StringUtils.equals(v, Constant.ReplaceKeyword.KEY_IN);
    }

    private void fixedAccountCode(PeriodCellTemplateConfig pctc) {

        if (StringUtils.isNotBlank(pctc.getAccountCodes())) {
            String[] acctCodes = pctc.getAccountCodes().split(",");
            EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
            enterpriseAccountExample.createCriteria().andStdCodeIn(Arrays.asList(acctCodes));

            List<EnterpriseAccount> enterpriseAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
            if (!enterpriseAccounts.isEmpty()) {
                List<String> acctCodeList = enterpriseAccounts.stream().map(EnterpriseAccount::getCode).collect(Collectors.toList());
                StringBuilder codes = new StringBuilder();
                for (String code : acctCodeList) {
                    codes.append(code).append(",");
                }
                pctc.setAccountCodes(StringUtils.removeEnd(codes.toString(), ","));
            }
        }
    }

    private void fixedParsedFormula(PeriodCellTemplateConfig pctc) {

        if (pctc.getFormula() != null && pctc.getFormula().contains("@")) {
            String regex = "@[0-9A-Z.]+";
            Pattern pp = Pattern.compile(regex);
            Matcher mm = pp.matcher(pctc.getFormula());
            String replace_result = pctc.getFormula();
            Boolean replace = true;
            while (mm.find()) {
                KeyValueConfigExample keyValueConfigExample = new KeyValueConfigExample();
                keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(mm.group(), "@"));
                Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
                if (keyValueConfig.isPresent() && StringUtils.isNotBlank(keyValueConfig.get().getFormula())) {
                    replace_result = replace_result.replace(mm.group(), keyValueConfig.get().getFormula());
                } else {
                    replace = false;
                }
            }

            if (replace) {
                pctc.setKeyValueParsedFormula(replace_result);
            }
            //todo: get the keyvalue from table add to keyvalue parsed formula
//                        KeyValueConfigExample skeyValueConfigExample = new KeyValueConfigExample();
//                        keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(cellTemplateConfig.getFormula(), "@"));
//                        Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
//                        if (keyValueConfig.isPresent()) {
//                            periodCellTemplateConfig.setKeyValueParsedFormula(keyValueConfig.get().getFormula());
//                        }
        }
    }

    private void copyPeriodCellTemplateFromCellTemplate(String projectId, Long templateId, Integer period) {
        CellTemplateExample cellTemplateExample = new CellTemplateExample();
        cellTemplateExample.createCriteria().andReportTemplateIdEqualTo(templateId);
        List<CellTemplate> cellTemplateList = cellTemplateMapper.selectByExample(cellTemplateExample);
        List<PeriodCellTemplate> periodCellTemplateList = new ArrayList<>();

        for (CellTemplate cellTemplate : cellTemplateList) {
            PeriodCellTemplate periodCellTemplate = new PeriodCellTemplate();
            CommonUtils.copyProperties(cellTemplate, periodCellTemplate);
            periodCellTemplate.setId(distributedIdService.nextId());
            periodCellTemplate.setPeriod(period);
            periodCellTemplate.setReportTemplateId(templateId);
            periodCellTemplate.setRowIndex(cellTemplate.getRowIndex());
            periodCellTemplate.setRowName(cellTemplate.getRowName());
            periodCellTemplate.setColumnIndex(cellTemplate.getColumnIndex());
            periodCellTemplate.setColumnName(cellTemplate.getColumnName());
            periodCellTemplate.setComment(cellTemplate.getComment());
            periodCellTemplate.setCreateTime(cellTemplate.getCreateTime());
            periodCellTemplate.setUpdateTime(cellTemplate.getUpdateTime());
            periodCellTemplate.setCellTemplateId(cellTemplate.getId());
            periodCellTemplate.setDataType(cellTemplate.getDataType());
            periodCellTemplate.setIsReadOnly(cellTemplate.getIsReadOnly() ? 1 : 0);
            periodCellTemplate.setCopyFromId(cellTemplate.getCopyFromId());
            periodCellTemplate.setCreateBy(StringUtils.isBlank(cellTemplate.getCreateBy()) ? "admin" : cellTemplate.getCreateBy());
            periodCellTemplate.setUpdateBy(StringUtils.isBlank(cellTemplate.getUpdateBy()) ? "admin" : cellTemplate.getUpdateBy());
            periodCellTemplate.setProjectId(projectId);
            periodCellTemplateList.add(periodCellTemplate);
        }

        CommonUtils.subListWithLen(periodCellTemplateList, CommonUtils.BATCH_NUM).forEach(batch -> {
            periodCellTemplateMapper.batchInsert(batch);

        });


    }

    private List<Template> queryTemplateByGroup(Long templateGroupId) {
        TemplateExample example = new TemplateExample();
        example.createCriteria().andTemplateGroupIdEqualTo(templateGroupId).andIsActiveAssociationEqualTo(true);
        return templateMapper.selectByExample(example);
    }

    private void copyTemplateAndConfigFromAdmin(String projectId, List<Template> templates, Integer period) {
        List<PeriodTemplate> periodTemplateList = new ArrayList<>();
        for (Template template : templates) {
            Long startTime = System.currentTimeMillis();
            logger.debug("template copy start: " + startTime);
            PeriodTemplate periodTemplate = new PeriodTemplate();
            CommonUtils.copyProperties(template, periodTemplate);
            periodTemplate.setId(distributedIdService.nextId());
            periodTemplate.setPeriod(period);
            periodTemplate.setName(template.getName());
            periodTemplate.setCode(template.getCode());
            periodTemplate.setPath(template.getPath());
            periodTemplate.setReportType(template.getReportType());
            periodTemplate.setTemplateGroupId(template.getTemplateGroupId());
            periodTemplate.setOrderIndex(template.getOrderIndex());
            periodTemplate.setCreateTime(template.getCreateTime());
            periodTemplate.setUpdateTime(template.getUpdateTime());
            periodTemplate.setIsSystemType(template.getIsSystemType());
            periodTemplate.setIsActiveAssociation(template.getIsActiveAssociation());
            periodTemplate.setParentId(StringUtils.isBlank(template.getParentId()) ? null : template.getParentId());
            periodTemplate.setTemplateId(template.getId());
            periodTemplate.setCreateBy(StringUtils.isBlank(template.getCreateBy()) ? "admin" : template.getCreateBy());
            periodTemplate.setUpdateBy(StringUtils.isBlank(template.getUpdateBy()) ? "admin" : template.getUpdateBy());
            periodTemplate.setProjectId(projectId);
            //TODO 当是税金计算表时动态添加行
            if ("VAT020".equals(periodTemplate.getCode())) {
                ByteArrayOutputStream bout = new ByteArrayOutputStream();
                InputStream is = null;
                try {
                    DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                    fileParam.setUuids(Arrays.asList(periodTemplate.getPath()));
                    PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                    Map<String, String> urlMap = null;
                    is = httpFileService.getUserTemplate(uploadDetail.getList().get(0).getViewHttpUrl());
                    Workbook tWorkbook = WorkbookFactory.create(is);
                    tWorkbook = assembleTaxWorkBook(template, tWorkbook, projectId, period);
                    tWorkbook.write(bout);
                    FileUpload fileUpload = didiFileUploadService.uploadFile(bout.toByteArray(), template.getCode() + "_" + template.getName() + ".xlsx", FileUploadEnum.BizSource.PERIOD_REPORT_TEMPLATE_UPLOAD.name());
                    periodTemplate.setPath(fileUpload.getUid());

                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        is.close();
                        bout.close();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            } else {
                copyPeriodCellTemplateFromCellTemplate(projectId, template.getId(), period);
                copyPeriodConfigFromCellTemplateConfig(projectId, template.getId(), period);
            }
            periodTemplateList.add(periodTemplate);
        }

        periodTemplateMapper.batchInsert(periodTemplateList);
    }

    private String transNumber(int num, String result) {

        if (num == 0) {
            return result;
        }
        int pre = num / 26;// 取商
        int res = num % 26;// 取余数
        if (res != 0) {
            result = (char) (res + 'A' - 1) + result;
        } else {
            result += 'Z';
            pre = pre - 1;
        }
        return transNumber(pre, result);
    }

    public Workbook assembleTaxWorkBook(Template template, Workbook tWorkbook, String projectId, Integer period) {
        Sheet sheet = tWorkbook.getSheetAt(0);
        Project project = projectMapper.selectByPrimaryKey(projectId);
        RevenueConfigExample example = new RevenueConfigExample();
        String queryDate = project.getYear() + "-" + (period >= 10 ? period : ("0" + period));
        example.createCriteria().andOrgIdEqualTo(project.getOrganizationId()).
                andStartDateLessThanOrEqualTo(queryDate).andEndDateGreaterThanOrEqualTo(queryDate).andStatusEqualTo(0);
        example.setOrderByClause(" create_time ");
        List<RevenueConfig> dataList = revenueConfigMapper.selectByExample(example);
        //合计项map
        Map<Integer, List<String>> sumMap = new HashMap<>();
        sumMap.put(TaxesCalculateReportEnum.Column.Column_4.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_5.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_6.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_7.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_8.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_9.getIndex(), new ArrayList<>());
        sumMap.put(TaxesCalculateReportEnum.Column.Column_11.getIndex(), new ArrayList<>());
        List<Long> configIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dataList)) {

            int rowIndex = 1;
            Row sourceRow = sheet.getRow(3);

            for (RevenueConfig config : dataList) {
                configIds.add(config.getId());
                Row row = POIUtil.createAndCloneRow(tWorkbook, sheet, 1, sourceRow);

                row.getCell(TaxesCalculateReportEnum.Column.Column_1.getIndex()).setCellValue("1-" + (dataList.size() + 1 - rowIndex));
                row.getCell(TaxesCalculateReportEnum.Column.Column_2.getIndex()).setCellValue(config.getName());
                row.getCell(TaxesCalculateReportEnum.Column.Column_3.getIndex()).setCellValue("PC(\"FilterDate1\",2)");
                //判断帐载收入明细
                if (0 == config.getAccountType()) {//0值
                    row.getCell(TaxesCalculateReportEnum.Column.Column_4.getIndex()).setCellValue(0.00);
                } else if (1 == config.getAccountType()) {//科目
                    row.getCell(TaxesCalculateReportEnum.Column.Column_4.getIndex()).setCellValue(
                            "DFFS(\"" + config.getTbSegment3() + "\"," + project.getYear() + "," + period + ",1,\"" + config.getTbSegment5() + "\",\"" + config.getTbSegment6() + "\")-"
                                    + "JFFS(\"" + config.getTbSegment3() + "\"," + project.getYear() + "," + period + ",1,\"" + config.getTbSegment5() + "\",\"" + config.getTbSegment6() + "\")");
                } else if (2 == config.getAccountType()) {//手工输入

                } else {
                    row.getCell(TaxesCalculateReportEnum.Column.Column_4.getIndex()).setCellValue("");
                }
                row.getCell(TaxesCalculateReportEnum.Column.Column_5.getIndex()).setCellValue("KPSR(\"" + config.getName() + "\",2,1)");
                row.getCell(TaxesCalculateReportEnum.Column.Column_6.getIndex()).setCellValue("KPSR(\"" + config.getName() + "\",2,2)");
                row.getCell(TaxesCalculateReportEnum.Column.Column_7.getIndex()).setCellValue("KPSR(\"" + config.getName() + "\",1,1)");
                row.getCell(TaxesCalculateReportEnum.Column.Column_8.getIndex()).setCellValue("KPSR(\"" + config.getName() + "\",1,2)");
                if (1 == config.getTaxBase()) {//账载
                    row.getCell(TaxesCalculateReportEnum.Column.Column_9.getIndex()).setCellValue("WPNAME(\"VAT020\",\"B\",\"" + config.getName() + "\",\"D\")");
                } else if (2 == config.getTaxBase()) {//开票收入
                    row.getCell(TaxesCalculateReportEnum.Column.Column_9.getIndex()).setCellValue("WPNAME(\"VAT020\",\"B\",\"" + config.getName() + "\",\"E\")+"
                            + "WPNAME(\"VAT020\",\"B\",\"" + config.getName() + "\",\"G\")");
                } else if (3 == config.getTaxBase()) {//手工录入

                } else if (4 == config.getTaxBase()) {//借方发生额
                    row.getCell(TaxesCalculateReportEnum.Column.Column_9.getIndex()).setCellValue("JFFS(\"" + config.getBaseDrCode() + "\"," + project.getYear() + "," + period + ",1,,)");
                } else if (5 == config.getTaxBase()) {//贷方发生额
                    row.getCell(TaxesCalculateReportEnum.Column.Column_9.getIndex()).setCellValue("DFFS(\"" + config.getBaseCrCode() + "\"," + project.getYear() + "," + period + ",1,,)");
                } else {
                    row.getCell(TaxesCalculateReportEnum.Column.Column_9.getIndex()).setCellValue("");
                }
                row.getCell(TaxesCalculateReportEnum.Column.Column_10.getIndex()).setCellValue(config.getTaxRate().multiply(new BigDecimal(100)).intValue() + "%");
                row.getCell(TaxesCalculateReportEnum.Column.Column_11.getIndex()).setCellValue("WPNAME(\"VAT020\",\"B\",\"" + config.getName() + "\",\"I\")*"
                        + "WPNAME(\"VAT020\",\"B\",\"" + config.getName() + "\",\"J\")");
                row.getCell(TaxesCalculateReportEnum.Column.Column_12.getIndex()).setCellValue(RevenueConfEnum.RevenueType.MAPPING.get(config.getRevenueType()));
                row.getCell(TaxesCalculateReportEnum.Column.Column_13.getIndex()).setCellValue(RevenueConfEnum.TaxType.MAPPING.get(config.getTaxType()));
                rowIndex++;
                //组装合计项
                sumMap.get(TaxesCalculateReportEnum.Column.Column_4.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_4.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_5.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_5.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_6.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_6.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_7.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_7.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_8.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_8.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_9.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_9.getIndex() + 1, "") + rowIndex);
                sumMap.get(TaxesCalculateReportEnum.Column.Column_11.getIndex()).add(transNumber(TaxesCalculateReportEnum.Column.Column_11.getIndex() + 1, "") + rowIndex);

            }
            //组装合计项数据
            assembleSumRow(sheet.getRow(rowIndex), sumMap);
        }
        assemblePeriodTemplate(template, tWorkbook, projectId, period, configIds);
        return tWorkbook;
    }

    public void assembleSumRow(Row sumRow, Map<Integer, List<String>> sumMap) {
        Iterator<Map.Entry<Integer, List<String>>> iterator = sumMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<Integer, List<String>> entry = iterator.next();
            String sumStr = "PSUM(\"";
            for (String cellNum : entry.getValue()) {
                sumStr += cellNum + ",";
            }
            sumStr += "\")";
            sumStr = sumStr.replace(",\")", "\")");

            sumRow.getCell(entry.getKey()).setCellValue(sumStr);
        }

    }

    public Workbook generateReportData(List<Long> reportIds, String projectId, Integer period) {
        PeriodReportExample reportExample = new PeriodReportExample();
        reportExample.createCriteria().andIdIn(reportIds);
        List<PeriodReport> reports = periodReportMapper.selectByExample(reportExample);
        PeriodTemplateExample example = new PeriodTemplateExample();
        example.createCriteria().andTemplateIdIn(reports.stream().map(m -> m.getTemplateId()).collect(Collectors.toList())).andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        List<PeriodTemplate> dataList = periodTemplateMapper.selectByExample(example);
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        Workbook tWorkbook = null;
        try {
            tWorkbook = reportGenerator.createWorkBookByPeriodTemplate(dataList);
            tWorkbook.write(bout);

//            FileUpload fileUpload = didiFileUploadService.uploadFile(bout.toByteArray(), "demo.xlsx", FileUploadEnum.BizSource.PERIOD_REPORT_TEMPLATE_UPLOAD.name());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                bout.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return tWorkbook;
    }

    ;

    public OperationResultDto generateData(String projectId, EnumServiceType serviceType, Boolean isMergeManualData,
                                           Integer periodParam, Integer reportType, Optional<String> generator) {
        OperationResultDto operationResultDto = new OperationResultDto();
        MyAsserts.assertEq(serviceType, EnumServiceType.VAT, new NotFoundException());
        MyAsserts.assertNull(periodJobMapper.getRunningJob(projectId, periodParam), Exceptions.TASK_HAS_BEGINNING);
        String status = periodApprovalMapper.getStatusByProjectIdAndPeriod(projectId, periodParam);
        MyAsserts.assertTrue(status == null || !status.equals(Constant.APPROVAL_COMMITTED), Exceptions.REPORT_IN_PROCESS_OR_AGREED_EXCEPTION);
        try {
            if (serviceType.equals(EnumServiceType.VAT) && (periodParam == null || periodParam <= 0)) {
                operationResultDto.setResultMsg("PeriodRequiredForVAT");
                return operationResultDto;
            }
            List<Template> templates = getTemplatesByProjectId(projectId);
            List<String> dataValidateArray = new ArrayList<String>();
            /*dataValidateArray.add("DA001");
            dataValidateArray.add("DA002");
            dataValidateArray.add("DA003");*/
            dataValidateArray.add("DA004");
            PeriodJob genJob = WrapPeriodJobDto.createReportGenJob(projectId, periodParam, templates, dataValidateArray);

            periodJobMapper.insert(genJob);
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        updateConfig(projectId, periodParam, isMergeManualData, templates, genJob);
                        //进行数据校验
                        DataValidation(periodParam, projectId, genJob);
                        PeriodResources resources = reportGenerator.getPeriodResources(projectId, periodParam,
                                templates.stream().map(Template::getId).collect(Collectors.toList()));

                        Workbook workbook = reportGenerator.createWorkBookByPeriodTemplate(resources.getPeriodTemplates(), genJob);

                        reportGenerator.addFunctionsAndContext(workbook, functions, reportGenerator.initContext(resources, periodParam), false);
                        reportGenerator.setConfigAndDataToWorkBook(workbook, resources);

                        FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
                        evaluator.evaluateAll();
                        reportGenerator.updateWorkbookCaclsValueToDb(projectId, periodParam, workbook, resources, isMergeManualData, genJob);
                        //===============================================start validation compute===========================================================================================================
                        //todo: 1.get the data from workbook, then put the data into new workbook
                        Workbook workbook4Validate = reportGenerator.createWorkBookByPeriodTemplate(resources.getPeriodTemplates(), genJob);
                        copyDataToWorkbook4Validate(workbook, workbook4Validate);
                        //todo: 2.get the validate formula in the cell_template_config
                        List<PeriodCellTemplateConfig> periodCellTemplateConfigs = resources.getPeriodCellTemplateConfigs().stream()
                                .filter(a -> a.getDataSourceType().equals(CellDataSourceType.Validation.getCode())).collect(Collectors.toList());
                        List<PeriodCellTemplateConfigDto> periodCellTemplateConfigDtos = new ArrayList<>();
                        periodCellTemplateConfigs.forEach(a -> {
                            if (StringUtils.isNotBlank(a.getValidation())) {
                                Optional<Template> template = templates.stream().filter(t -> t.getId().equals(a.getReportTemplateId())).findFirst();
                                String templateCode = template.isPresent() ? template.get().getCode() : "";
                                Integer sheetNumber = template.isPresent() ? template.get().getOrderIndex() : null;
                                Optional<PeriodCellTemplate> periodCellTemplate = resources.getPeriodCellTemplates().stream().filter(c -> c.getCellTemplateId().equals(a.getCellTemplateId())).findFirst();
                                Integer rowNumber = periodCellTemplate.isPresent() ? periodCellTemplate.get().getRowIndex() : null;
                                Integer colNumber = periodCellTemplate.isPresent() ? periodCellTemplate.get().getColumnIndex() : null;
                                if (StringUtils.isNotBlank(templateCode) && rowNumber != null && colNumber != null) {
                                    String replaceFormula = String.format("BB(\"%s\",%d,%d,0,0)", templateCode, colNumber, rowNumber);
                                    a.setParsedValidation(a.getValidation().replace("aa()", replaceFormula));
                                    PeriodCellTemplateConfigDto periodCellTemplateConfigDto = new PeriodCellTemplateConfigDto();
                                    periodCellTemplateConfigDto.setSheetNumber(sheetNumber);
                                    periodCellTemplateConfigDto.setRowNumber(rowNumber);
                                    periodCellTemplateConfigDto.setColNumber(colNumber);
                                    periodCellTemplateConfigDto.setValidation(a.getValidation());
                                    periodCellTemplateConfigDto.setParsedValidation(a.getParsedValidation());
                                    periodCellTemplateConfigDto.setCellTemplateId(periodCellTemplate.get().getCellTemplateId());
                                    periodCellTemplateConfigDto.setProjectId(projectId);
                                    periodCellTemplateConfigDto.setPeriodParam(periodParam);
                                    periodCellTemplateConfigDtos.add(periodCellTemplateConfigDto);
                                }
                            }
                        });
                        //todo: 3.use the validate formula to recalculate the new workbook
                        periodCellTemplateConfigDtos.forEach(a -> {
                            workbook4Validate.getSheetAt(a.getSheetNumber() - 1).getRow(a.getRowNumber()).getCell(a.getColNumber()).setCellFormula(a.getParsedValidation());
                        });
                        reportGenerator.addFunctionsAndContext(workbook4Validate, functions, reportGenerator.initContext(resources, periodParam), true);
                        FormulaEvaluator validateEvaluator = workbook4Validate.getCreationHelper().createFormulaEvaluator();
                        validateEvaluator.evaluateAll();
                        //todo: 4.then save the validation result to cellData table
                        periodCellTemplateConfigDtos.forEach(a -> {
                            //todo: according to periodParam , projectId, cellTemplateId to update the cellData table
                            PeriodCellDataExample periodCellDataExample = new PeriodCellDataExample();
                            periodCellDataExample.createCriteria().andCellTemplateIdEqualTo(a.getCellTemplateId()).andProjectIdEqualTo(a.getProjectId()).andPeriodEqualTo(a.getPeriodParam());
                            Optional<PeriodCellData> cellData = periodCellDataMapper.selectByExample(periodCellDataExample).stream().findFirst();
                            if (cellData.isPresent()) {
                                cellData.get().setValidateFormulaExp(a.getParsedValidation());
                                Boolean result = workbook4Validate.getSheetAt(a.getSheetNumber() - 1).getRow(a.getRowNumber()).getCell(a.getColNumber()).getBooleanCellValue();
                                cellData.get().setValidateResult(result.toString());
                                periodCellDataMapper.updateByPrimaryKey(cellData.get());
                            }
                        });
                        //===============================================end validation compute============================================================================================================
                        setStatus(genJob, STATUS_END);
                        periodJobMapper.updateByPrimaryKey(genJob);
                    } catch (Exception e) {
                        setStatus(genJob, STATUS_ERROR);
                        e.printStackTrace();
                        genJob.setErrorMsg("Sever error " + e.getClass().getName());
                        periodJobMapper.updateByPrimaryKey(genJob);
                    }
                }

            }).start();
            operationResultDto.setData(new PeriodJobDto().copyFromPeriodJob(genJob));
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            operationResultDto.setResult(false);
            logger.error(ex.getMessage(), ex);
        }
        return operationResultDto;
    }

    private void copyDataToWorkbook4Validate(Workbook workbook, Workbook workbook4Validate) {
        int sheetNumber = workbook.getNumberOfSheets();
        for (int i = 0; i < sheetNumber; i++) {
            Sheet sheet = workbook.getSheetAt(i);
            for (Row row : sheet) {
                for (Cell cell : row) {
                    String data;
                    String result;
                    data = ((XSSFCell) cell).getRawValue();
                    if (data != null && data.equals("#VALUE!")) {
                        FormulaEvaluator formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
                        try {
                            data = formulaEvaluator.evaluate(cell).getStringValue();
                        } catch (Exception e) {
                            logger.error(e.getStackTrace().toString());
                            data = "0.0";
                        }
                    }

                    if (StringUtils.isNotBlank(data)) {
                        Pattern pattern = Pattern.compile("[0-9.]*");
                        Matcher isNum = pattern.matcher(data);
                        if (isNum.matches()) {
                            result = new BigDecimal(data).toString();
                        } else {
                            result = data;
                        }
                    } else {
                        result = data;
                    }
                    workbook4Validate.getSheetAt(i).getRow(row.getRowNum()).getCell(cell.getColumnIndex()).setCellValue(result);
                }
            }
        }
    }

    @Resource
    private TrialBalanceMapper trialBalanceMapper;
    @Resource
    private AdjustmentTableMapper adjustmentTableMapper;
    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;
    @Resource
    private JournalEntryMapper journalEntryMapper;
    @Autowired
    private DataUtil dataUtil;

    /*  @Autowired
      private CitJour*/
    //数据校验
    private void DataValidation(Integer periodParam, String projectId, PeriodJob genJob) {
        setStatus(genJob, "DA004", STATUS_BEGIN);

        //本期余额表
      /*  TrialBalanceExample trialBalanceExample = new TrialBalanceExample();
        TrialBalanceExample.Criteria criteria = trialBalanceExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        criteria.andPeriodEqualTo(periodParam);
        TrialBalance trialBalances = trialBalanceMapper.selectByExample(trialBalanceExample).get(0);
        //本期利润表
        ProfitLossStatementExample profitLossStatementExample = new ProfitLossStatementExample();
        ProfitLossStatementExample.Criteria criteria2 = profitLossStatementExample.createCriteria();
        criteria2.andProjectIdEqualTo(projectId);
        criteria2.andPeriodEqualTo(periodParam);
        ProfitLossStatement profitLossStatements = profitLossStatementMapper.selectByExample(profitLossStatementExample).get(0);*/


        //调整表
        Map<String, Object> map1 = new HashedMap();
        map1.put("projectId", projectId);
        map1.put("period", periodParam);
        if ((periodParam + "").length() == 1) {
            map1.put("period", "0" + periodParam);
        } else {
            map1.put("period", "" + periodParam);
        }

        List<AdjustmentTable> adjustmentTables = adjustmentTableMapper.selectBeforeAdjustData(map1);
        Map<String, Object> map2 = new HashedMap();
        map2.put("projectId", projectId);
        if ((periodParam + "").length() == 1) {
            map2.put("period", "0" + periodParam);
        } else {
            map2.put("period", "" + periodParam);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("period", periodParam);
        map.put("projectId", projectId);
        Map<String, Object> mapProject = dataUtil.getProjectById(projectId);
        map.put("companyCode", mapProject.get("code"));
        map.put("companyName", mapProject.get("name"));
        List<JournalEntry> journalEntries = journalEntryMapper.selectNowAdjustData(map2);
        for (int i = 0; i < adjustmentTables.size(); i++) {
            for (int j = 0; j < journalEntries.size(); j++) {
                JournalEntry journalEntry = journalEntries.get(j);
                AdjustmentTable adjustmentTable = adjustmentTables.get(j);
                boolean boo = ("".equals(journalEntry.getSegment3()) && "".equals(adjustmentTable.getSegment3()) && "".equals(journalEntry.getSegment5()) && "".equals(adjustmentTable.getSegment5()) && "".equals(journalEntry.getSegment6()) && "".equals(adjustmentTable.getSegment6()));
                if (journalEntry.getSegment3().equals(adjustmentTable.getSegment3()) &&
                        journalEntry.getSegment5().equals(adjustmentTable.getSegment5()) &&
                        journalEntry.getSegment6().equals(adjustmentTable.getSegment6()) && journalEntry.getPeriodJrMinDr() != adjustmentTable.getPeriodDrMixCr() && !boo) {
                    setStatus(genJob, STATUS_ERROR);
                    Constant.ReportDataValidateLog reportDataValidateLog = new Constant.ReportDataValidateLog(journalEntries.get(j).getSegment3(), journalEntries.get(j).getSegment5(), journalEntries.get(j).getSegment6());
                    //记录校验结果
                    map.put("result", reportDataValidateLog.getResult(false));
                    map.put("validateResult", "error");
                    map.put("tmsPeriod", journalEntry.getTmsPeriod());
                    map.put("organizationId", journalEntry.getOrganizationId());

                    insertDataValidateResult(map);
                    periodJobMapper.updateByPrimaryKey(genJob);
                    return;
                }
            }
        }
        setStatus(genJob, STATUS_END);
        map.put("validateResult", "success");
        map.put("result", "");
        if (journalEntries.size() == 0) {
            map.put("tmsPeriod", 0);

        } else {
            map.put("tmsPeriod", journalEntries.get(0).getTmsPeriod());
        }
        map.put("organizationId", mapProject.get("organizationId"));

        insertDataValidateResult(map);
        periodJobMapper.updateByPrimaryKey(genJob);
    }


    @Resource
    private DataValidateLogMapper dataValidateLogMapper;

    public void insertDataValidateResult(Map map) {
        DataValidateLog dataValidateLog = new DataValidateLog();
        dataValidateLog.setCreateTime(new Date());
        dataValidateLog.setPeriod((Integer) map.get("period"));
        dataValidateLog.setProjectId((String) map.get("projectId"));
        dataValidateLog.setValidateResult((String) map.get("validateResult"));
        dataValidateLog.setResult((String) map.get("result"));
        dataValidateLog.setOrganizationId((String) map.get("organizationId"));
        dataValidateLog.setCompanyCode((String) map.get("companyCode"));
        dataValidateLog.setCompanyName((String) map.get("companyName"));
        dataValidateLog.setTmsPeriod((Integer) map.get("tmsPeriod"));
        dataValidateLogMapper.insert(dataValidateLog);
    }


    private String getProjId(String orgId, Integer tmsPeriod) {
        ProjectExample pExample = new ProjectExample();
        pExample.createCriteria().andOrganizationIdEqualTo(orgId).andYearEqualTo(tmsPeriod / 100);
        List<Project> pList = projectMapper.selectByExample(pExample);
        if (!pList.isEmpty()) {
            return pList.get(0).getId();
        }
        return "";
    }

    private static final Integer specialConsideration = 0;//特殊因素考虑

    public List<CellTemplateReferenceDto> getTemplateReferences(int period) {
        return new ArrayList<>();
    }

    public OperationResultDto<ReportDataDto> getCellData(Long reportId, String orgId, Integer period) {
        OperationResultDto<ReportDataDto> operationResultDto = new OperationResultDto<>();
        OperationResultDto<ReportDataDto> operationResultDto1 = loadEbitDataFirst(orgId, period, operationResultDto);//从数据库中加载ebit数据
        if (operationResultDto1.getData().getEbitData() != null && operationResultDto1.getData().getCellData() != null) {
            return operationResultDto1;
        } else if (operationResultDto1.getData().getEbitData() == null && operationResultDto1.getData().getCellData() == null) {
            OperationResultDto<ReportDataDto> cellData = getCellData(reportId, getProjId(orgId, period));
            ReportDataDto reportDataDto = new ReportDataDto();
            //特殊因素考虑 默认为0 rate 默认为1%
            EbitDataDto ebitDataDto1 = loadEbitData(orgId, period, specialConsideration, Constant.ebitRate);
            if (cellData.getData() != null) {
                cellData.getData().setEbitData(ebitDataDto1);
            } else {

                reportDataDto.setEbitData(ebitDataDto1);
                cellData.setData(reportDataDto);
                return cellData;
            }
        } else if (operationResultDto1.getData().getEbitData() == null && operationResultDto1.getData().getCellData() != null) {
            EbitDataDto ebitDataDto1 = loadEbitData(orgId, period, specialConsideration, Constant.ebitRate);
            operationResultDto1.getData().setEbitData(ebitDataDto1);
        } else {
            OperationResultDto<ReportDataDto> cellData = getCellData(reportId, getProjId(orgId, period));
            operationResultDto1.getData().setCellData(cellData.getData().getCellData());
        }
        return operationResultDto1;
    }

    public OperationResultDto loadEbitDataFirst(String orgId, Integer period, OperationResultDto operationResultDto) {
        EbitDataDto ebitDataDto = new EbitDataDto();
        boolean bool = false;
        boolean boo2 = false;
        EbitCellDataExample example = new EbitCellDataExample();
        EbitCellDataExample.Criteria criteria = example.createCriteria();
        criteria.andOrganizationIdEqualTo(orgId);
        ReportDataDto reportDataDto = new ReportDataDto();
        criteria.andPeriodEqualTo(period);
        List<EbitCellData> ebitCellData = ebitCellDataMapper.selectByExample(example);
        ArrayList<CellDataDto> listCelldatas = Lists.newArrayList();
        for (EbitCellData ebitCellData1 : ebitCellData) {
            bool = true;
            switch (ebitCellData1.getRow()) {
                case 37:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setSpecialConsiderations(ebitCellData1.getData());
                    break;
                case 38:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setSpecialConsiderations(ebitCellData1.getData());
                    break;
                case 39:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setSpecialFactors(new BigDecimal(ebitCellData1.getData()));
                    break;
                case 40:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setEbitRate(ebitCellData1.getData());
                    break;
                case 41:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setTransactionAmount(new BigDecimal(ebitCellData1.getData()));
                    break;
                case 42:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setSixAddTax(new BigDecimal(ebitCellData1.getData()));
                    break;
                case 43:
                    if (ebitCellData1.getData() == null)
                        break;
                    ebitDataDto.setTotalAmountTax(new BigDecimal(ebitCellData1.getData()));
                    break;
                default:
                    if (ebitCellData1.getRow() < 37) {
                        if (!boo2)
                            boo2 = true;
                        CellDataDto cellData = new CellDataDto();
                        cellData.setColumnIndex(ebitCellData1.getCol());
                        cellData.setRowIndex(ebitCellData1.getRow());
                        cellData.setCellValue(ebitCellData1.getData());
                        listCelldatas.add(cellData);
                    }

            }
        }

        reportDataDto.setCellData(listCelldatas);
        reportDataDto.setEbitData(ebitDataDto);
        if (!bool)
            reportDataDto.setEbitData(null);
        if (!boo2)
            reportDataDto.setCellData(null);
        operationResultDto.setData(reportDataDto);
        return operationResultDto;
    }

    public OperationResultDto<ReportDataDto> getCellData(Long reportId, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        ReportDataDto dataDto = new ReportDataDto();
        try {
            PeriodReport report = periodReportMapper.selectByPrimaryKey(reportId);
            if (report == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReport");
                return resultDto;
            }

            PeriodTemplateExample example = new PeriodTemplateExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(report.getTemplateId()).andPeriodEqualTo(report.getPeriod());
            Optional<PeriodTemplate> reportTemplate = periodTemplateMapper.selectByExample(example).stream().findFirst();
            if (!reportTemplate.isPresent()) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReportTemplate");
                return resultDto;
            }

            Project project = projectMapper.selectByPrimaryKey(report.getProjectId());
            if (project == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoProject");
                return resultDto;
            }

            PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
            periodTemplateExample.createCriteria()
                    .andProjectIdEqualTo(projectId)
                    .andTemplateGroupIdEqualTo(reportTemplate.get().getTemplateGroupId())
                    .andPeriodEqualTo(report.getPeriod())
                    .andIsActiveAssociationEqualTo(true);

            List<Long> templateIds = new ArrayList<>();
            templateIds.add(report.getTemplateId());
            List<CellCalcInfoDto> cellCfgList = reportGenerator.getCellCalcInfo(templateIds, report.getPeriod(), project.getId());

            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
            cellDataExample.createCriteria().andPeriodEqualTo(report.getPeriod()).andProjectIdEqualTo(projectId).andReportIdEqualTo(reportId);

            List<PeriodCellData> currentCellDataList = periodCellDataMapper.selectByExample(cellDataExample); //这是取的是单元格中显示的值
            PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
            periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(report.getPeriod())
                    .andReportIdEqualTo(reportId);
         /*   Map<String, Object> map = new HashMap<>();
            map.put("projectId", projectId);
            map.put("period", report.getPeriod());
            map.put("reportId", reportId);
            */
            List<PeriodFormulaBlock> formulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample);


            dataDto.setFormulaBlocks(formulaBlocks.stream()
                    .map(o -> beanUtil.copyProperties(o, new FormulaFormlaBlockDto())).collect(Collectors.toList()));
            //todo: 关键字数据源稍后再加
            // 获取通过KeyValueDataId关联的数据源，即公式数据源

            //todo: 模型数据源稍后再加
            // 获取通过ModelBackFillAmount关联的数据源，即模型数据源
            // TODO: Check模型回填数据源是否会有多对多的情况出现，如有需distinct

            //公式计算数据源 20180711 完成
            List<DataSourceExtendDto> manualDataSourceList = periodDataSourceMapper.getFormulaDataSource(reportId);
            //loop the datasource to add datasource detail information
            for (DataSourceExtendDto dseo : manualDataSourceList) {
                //只处理有明细数据的DataSource
                if (dseo.getType().equals(FormulaDataSourceType.InputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    //todo: add json datasource detail @ here. not only itemValue
                    List<String> itemValues = new ArrayList<>();
                    itemValues.add(getDataSourceDetailList(dseo.getId()).getData());
                    dseo.setItems(itemValues);
                } else if (dseo.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    List<String> itemValues = new ArrayList<>();
                    //todo: get the datasource detail to add the items property
                    PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
                    dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(dseo.getId());
                    List<PeriodDataSourceDetail> dataSourceDetailList = SpringContextUtil.periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
                    dataSourceDetailList.forEach(a -> {
                        if (StringUtils.isNotBlank(a.getItemValue())) {
                            itemValues.add(a.getItemValue());
                        }
                    });
                    if (dataSourceDetailList.size() > 0 && dataSourceDetailList.get(0).getDataSourceType() != 6) {
                        dseo.setDataSourceType(convertType(dataSourceDetailList.get(0).getDataSourceType()));
                    }
                    dseo.setItems(itemValues);
                }
            }

            List<DataSourceDtoExtend> datasource = new ArrayList<>();
            manualDataSourceList.forEach(a -> {
                DataSourceDto dataSourceDto = new DataSourceDto();
                dataSourceDto.setId(a.getId().toString());
                dataSourceDto.setAmount(a.getAmount());
                dataSourceDto.setKeyinData(a.getKeyinData());
                dataSourceDto.setName(a.getName());
                dataSourceDto.setOperationType(a.getOperationType());
                dataSourceDto.setDescription(a.getDescription());
                dataSourceDto.setCreateTime(a.getCreateTime());
                if (a.getDataSourceType() != null && a.getDataSourceType() > 0) {
                    dataSourceDto.setType(convetToFType(a.getDataSourceType()));
                } else {
                    dataSourceDto.setType(a.getType());
                }
                dataSourceDto.setItems(a.getItems());
                dataSourceDto.setPeriod(a.getPeriod());
                dataSourceDto.setRowIndex(a.getRowIndex());
                dataSourceDto.setRowName(a.getRowName());
                dataSourceDto.setColumnIndex(a.getColumnIndex());
                dataSourceDto.setColumnName(a.getColumnName());
                dataSourceDto.setReportTemplateId(a.getReportTemplateId().toString());
                dataSourceDto.setRel_sql(a.getRelSql());

                if (a.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    if (a.getItems().size() == 0) {
                        dataSourceDto.setDataSourceType(CellDataSourceType.Formula.getCode());
                    } else {
                        dataSourceDto.setDataSourceType(a.getDataSourceType());
                    }
                } else if (a.getType().equals(FormulaDataSourceType.KeyInSource.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.KeyIn.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.InputInvoice.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.Voucher.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.OutputInvoice.getCode());
                } else {
                    dataSourceDto.setDataSourceType(0);
                }

                DataSourceDtoExtend dataSourceDtoExtend = new DataSourceDtoExtend();
                dataSourceDtoExtend.setItem1(a.getCellTemplateId().toString());
                dataSourceDtoExtend.setItem2(dataSourceDto);

                datasource.add(dataSourceDtoExtend);
            });
            dataDto.setManualDataSources(datasource);

            List<CellDataDto> cellDataDtoList = new ArrayList<>();
            cellCfgList.forEach(a -> {
                CellDataDto cellDataDto = new CellDataDto();
                cellDataDto.setColumnIndex(a.getColumnIndex());
                cellDataDto.setColumnName(a.getColumnName());
                cellDataDto.setRowIndex(a.getRowIndex());
                cellDataDto.setRowName(a.getRowName());
                cellDataDto.setCellTemplateId(a.getCellTemplateId().toString());
                cellDataDto.setReportTemplateId(a.getReportTemplateId().toString());
                cellDataDto.setDataType(a.getDataType());
                cellDataDto.setReportId(reportId.toString());
                cellDataDto.setIsReadOnly(a.getIsReadOnly());
                Optional<PeriodCellData> celldata = currentCellDataList.stream()
                        .filter(x -> x.getCellTemplateId().toString().equals(a.getCellTemplateId())).findFirst();
                celldata.ifPresent(cellData -> cellDataDto.setCellId(cellData.getId().toString()));
                cellDataDto.setFormula(a.getFormula());
                celldata.ifPresent(cellData -> cellDataDto.setCellValue(cellData.getData()));
                celldata.ifPresent(cellData -> cellDataDto.setKeyinData(cellData.getKeyinData()));
                celldata.ifPresent(cellData -> cellDataDto.setFormulaExp(cellData.getFormulaExp()));
                celldata.ifPresent(cellData -> cellDataDto.setValidationResult(cellData.getValidateResult()));
                cellDataDto.setCellTemplateConfig(CellConfigTranslater.getConfigDto(a));
                List<DataSourceDtoExtend> entryList = datasource.stream()
                        .filter(y -> y.getItem1().equals(a.getCellTemplateId())).collect(Collectors.toList());
                List<DataSourceDto> dataSourceDtoList = new ArrayList<>();
                entryList.forEach(z -> {
                    if (z.getItem2().getItems() != null && !z.getItem2().getItems().isEmpty() && z.getItem2().getItems().get(0).contains("tag")) {
                        z.getItem2().getItems().forEach(m -> {
                            ReportCellDataSourceDto dto = JSON.parseObject(m, ReportCellDataSourceDto.class);
                            if (dto != null && dto.getReportTemplateId() != null && !Pattern.compile("\\D").matcher(dto.getReportTemplateId()).find()) {
                                PeriodTemplateExample periodTemplateExample1 = new PeriodTemplateExample();
                                periodTemplateExample1.createCriteria().andTemplateIdEqualTo(Long.parseLong(dto.getReportTemplateId()))
                                        .andPeriodEqualTo(report.getPeriod());
                                Optional<PeriodTemplate> optional = periodTemplateMapper.selectByExample(periodTemplateExample1).stream().findFirst();
                                if (optional.isPresent()) {
                                    dto.setReportName(optional.get().getName());
                                }
                            }

                            dataSourceDtoList.add(dto);
                        });
                    } else
                        dataSourceDtoList.add(z.getItem2());
                });
                cellDataDto.setDataSourceList(dataSourceDtoList);
                cellDataDtoList.add(cellDataDto);
            });

            PeriodModifiedReportCellExample modifiedReportCellExample = new PeriodModifiedReportCellExample();
            modifiedReportCellExample.createCriteria().andReportIdEqualTo(reportId);
            List<PeriodModifiedReportCell> modifiedReportCellList = periodModifiedReportCellMapper.selectByExample(modifiedReportCellExample);
            modifiedReportCellList.forEach(mrc -> {
                Optional<CellDataDto> cdd = cellDataDtoList.stream()
                        .filter(cellDataDto -> cellDataDto.getReportId().equals(mrc.getReportId())
                                && cellDataDto.getRowIndex() == mrc.getRow()
                                && cellDataDto.getColumnIndex() == mrc.getCol()).findFirst();
                if (cdd.isPresent()) {
                    if (!mrc.getOriginalValue().equals(mrc.getValue())) {
                        cdd.get().setModifiedReportCell(mrc);
                        cdd.get().setCellValue(mrc.getValue());
                        cdd.get().setIsModified(true);
                    }
                }
            });
            dataDto.setCellData(cellDataDtoList);
            //将cellData复制给静态变量
            _cellData = cellDataDtoList;
            resultDto.setData(dataDto);
            resultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
        }
        return resultDto;
    }

    @Resource
    private ProfitLossStatementFinalMapper profitLossStatementFinalMapper;
    @Resource
    private EbitCellDataMapper ebitCellDataMapper;


    //加载Ebit数据
    public EbitDataDto loadEbitData(String orgId, Integer period, Integer specialConsiderations, String ebitRate) {
        BigDecimal spec = new BigDecimal(0);
        double rate = 0.01; //ebitRate
        if (!ebitRate.isEmpty()) {
            int _index = ebitRate.indexOf("%");
            spec = new BigDecimal(specialConsiderations);

            if (_index != -1) {
                rate = Double.parseDouble(ebitRate.substring(0, _index)) / 100;
            } else {
                rate = Double.parseDouble(ebitRate) / 100;
            }
        }
        ProfitLossStatementExample example = new ProfitLossStatementExample();
        ProfitLossStatementExample.Criteria criteria = example.createCriteria();
        criteria.andOrganizationIdEqualTo(orgId);
        criteria.andPeriodEqualTo(period);
        List<ProfitLossStatement> profitLossStatements = profitLossStatementFinalMapper.selectByExample(example);
        BigDecimal yysr = new BigDecimal(0);//营业收入
        BigDecimal yycb = new BigDecimal(0);//营业成本
        BigDecimal yysjfj = new BigDecimal(0);//营业税金附加
        BigDecimal xsfy = new BigDecimal(0);//销售费用
        BigDecimal glfy = new BigDecimal(0);//管理费用
        BigDecimal yffy = new BigDecimal(0);//研发费用
        BigDecimal zcjzss = new BigDecimal(0); //资产减值损失
        for (ProfitLossStatement profitLossStatement : profitLossStatements) {
            BigDecimal value = profitLossStatement.getYtdAmt();
            switch (profitLossStatement.getItemName()) {
                case "一、营业收入":
                    yysr = value;
                    break;
                case "减：营业成本":
                    yycb = value;
                case "营业税金及附加":
                    yysjfj = value;
                    break;
                case "销售费用":
                    xsfy = value;
                case "管理费用":
                    glfy = value;
                    break;
                case "研发费用 ":
                    yffy = value;
                case "资产减值损失":
                    zcjzss = value;
            }
            EbitDataDto ebitDataDto = new EbitDataDto();
            BigDecimal subtract = yysr.subtract(yycb).add(yysjfj).add(xsfy).add(glfy).add(yffy).add(zcjzss);
            ebitDataDto.setEbitSubtraction(subtract);
            ebitDataDto.setSpecialFactors(subtract.add(spec));
            ebitDataDto.setEbitRate(String.valueOf(rate*100 + "%"));
            ebitDataDto.setTransactionAmount(ebitDataDto.getSpecialFactors().multiply(new BigDecimal(ebitDataDto.getEbitRate())));
            ebitDataDto.setEbitRate(ebitRate);
            ebitDataDto.setSixAddTax(ebitDataDto.getTransactionAmount().multiply(new BigDecimal(0.06)));
            ebitDataDto.setTotalAmountTax(ebitDataDto.getTransactionAmount().multiply(new BigDecimal(0.16)));
            return ebitDataDto;
        }
        return null;
    }

    private Integer convertType(Integer dataSourceType) {
        if (dataSourceType.equals(FormulaDataSourceDetailType.InputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.InputInvoice.getCode();
        } else if (dataSourceType.equals(FormulaDataSourceDetailType.OutputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.OutputInvoice.getCode();
        } else {
            return CellDataSourceType.Formula.getCode();
        }
    }

    private Integer convetToFType(Integer cellDataSourceType) {
        if (cellDataSourceType.equals(CellDataSourceType.InputInvoice.getCode())) {
            return FormulaDataSourceType.InputInvoice.getCode();
        }
        if (cellDataSourceType.equals(CellDataSourceType.OutputInvoice.getCode())) {
            return FormulaDataSourceType.OutputInvoice.getCode();
        } else {
            return FormulaDataSourceType.Other.getCode();
        }
    }

    public OperationResultDto<ReportDto> getReportByTemplate(Long templateId, Integer period, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            PeriodReportExample example = new PeriodReportExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(period);
            Optional<PeriodReport> report = periodReportMapper.selectByExample(example).stream().findFirst();
            if (report.isPresent()) {
                resultDto.setResult(true);
                ReportDto reportDto = new ReportDto();
                CommonUtils.copyProperties(report, reportDto);
                resultDto.setData(reportDto);
            } else {
                resultDto.setResult(false);
                resultDto.setResultMsg("can not find report");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            resultDto.setResult(false);
        }
        return resultDto;
    }

    /*
    *
     * EbitCellDataExample example1 = new EbitCellDataExample();
                EbitCellDataExample.Criteria criteria = example1.createCriteria();
                criteria.andPeriodEqualTo(period);
                criteria.andProjectIdEqualTo(projectId);
                criteria.andTemplateIdEqualTo(templateId.toString());
                List<EbitCellData> ebitCellData = ebitCellDataMapper.selectByExample(example1);
                if (!(ebitCellData.size() == 0)) {
                    report.get().setCellDataList(ebitCellData);
                }
                */
    public OperationResultDto<PeriodReportDto> getReportByTemplateEbit(Long templateId, Integer period, String projectId) {
        projectId = "1942295d-6fe6-4aa7-b2e7-b66a5772697c";
        period = 0;
        OperationResultDto resultDto = new OperationResultDto();
        try {
            PeriodReportExample example = new PeriodReportExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(period);
            Optional<PeriodReport> report = periodReportMapper.selectByExample(example).stream().findFirst();
            if (report.isPresent()) {
                PeriodReportDto periodReportDto = new PeriodReportDto();
                BeanUtils.copyProperties(report.get(), periodReportDto);
                resultDto.setResult(true);
                resultDto.setData(periodReportDto);
            } else {
                resultDto.setResult(true);
                resultDto.setResultMsg("can not find report");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            resultDto.setResult(false);
        }
        return resultDto;
    }

    public OperationResultDto<PeriodCellTemplateConfig> getCellTemplateConfig(Long reportTemplateId, Integer periodParam,
                                                                              int rowIndex, int columnIndex, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            int period = periodParam != null ? periodParam : 0;

            if (reportTemplateId == null || reportTemplateId == 0L) {
                resultDto.setResult(false);
                resultDto.setResultMsg("report template id is invalid");
                return resultDto;
            }

            PeriodCellTemplateConfig periodCellTemplateConfig = periodCellTemplateConfigMapper
                    .getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex, projectId);
            if (periodCellTemplateConfig == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("no config for this template!");
                return resultDto;
            } else {
                resultDto.setResult(true);
                resultDto.setData(periodCellTemplateConfig);
                return resultDto;
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
            return resultDto;
        }
    }

    public OperationResultDto<List<VatEnterpriseAccount>> getStdAccountByIndustry(String industryId, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            PeriodEnterpriseAccountExample example = new PeriodEnterpriseAccountExample();
            example.createCriteria().andIsActiveEqualTo(true).andProjectIdEqualTo(projectId);
            periodEnterpriseAccountMapper.selectByExample(example);
            List<PeriodEnterpriseAccount> vatEnterpriseAccounts = periodEnterpriseAccountMapper.selectByExample(example);
            if (vatEnterpriseAccounts == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("there is no data");
                return resultDto;
            } else {
                resultDto.setResult(true);
                resultDto.setData(vatEnterpriseAccounts);
                return resultDto;
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
            return resultDto;
        }
    }

    public OperationResultDto<List<VatEnterpriseAccountResultDto>> getCellAccountRange(Long reportTemplateId, Integer periodParam, int rowIndex, int columnIndex, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();

        try {
            int period = periodParam == null ? 0 : periodParam;

            PeriodCellTemplateConfig periodCellTemplateConfig =
                    periodCellTemplateConfigMapper.getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex, projectId);
            if (periodCellTemplateConfig == null || StringUtils.isBlank(periodCellTemplateConfig.getAccountCodes())) {
                resultDto.setResult(false);
                return resultDto;
            }

            PeriodEnterpriseAccountExample example = new PeriodEnterpriseAccountExample();
            example.createCriteria().andAcctCodeIn(Arrays.asList(periodCellTemplateConfig.getAccountCodes().split(","))).andProjectIdEqualTo(projectId);
            List<PeriodEnterpriseAccount> vatEnterpriseAccounts = periodEnterpriseAccountMapper.selectByExample(example);
            List<VatEnterpriseAccountResultDto> vatEnterpriseAccountResultDtos = new ArrayList<>();
            for (PeriodEnterpriseAccount vatEnterpriseAccount : vatEnterpriseAccounts) {
                VatEnterpriseAccountResultDto vatEnterpriseAccountResultDto = new VatEnterpriseAccountResultDto();
                vatEnterpriseAccountResultDto.setPeriodCellTemplateConfigId(periodCellTemplateConfig.getCellTemplateConfigId());
                vatEnterpriseAccountResultDto.setAcctCode(vatEnterpriseAccount.getAcctCode());
                vatEnterpriseAccountResultDto.setAcctPropName(vatEnterpriseAccount.getName());
                vatEnterpriseAccountResultDto.setDirectionName(vatEnterpriseAccount.getDirection() == 1 ? "借方" : "贷方");
                vatEnterpriseAccountResultDto.setAcctPropName(getAcctProp(vatEnterpriseAccount.getAcctProp()));
                vatEnterpriseAccountResultDtos.add(vatEnterpriseAccountResultDto);
            }

            resultDto.setData(vatEnterpriseAccountResultDtos);
            resultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
        }
        return resultDto;
    }

    public OperationResultDto addCellManualDataSource(ManualDataSourceDto data, String projectId) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            if (data.getCellId() == null) {
                //todo: insert celldata for manual datasource
                PeriodCellData cellData = new PeriodCellData();
                cellData.setId(distributedIdService.nextId());
                cellData.setReportId(data.getReportId());
                cellData.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellData.setData(data.getAmount() == null ? null : data.getAmount().toString());
                cellData.setKeyinData(data.getKeyinData());
                cellData.setFormulaExp(data.getAmount() == null ? "0.0" : data.getAmount().toString());
                cellData.setCreateBy("admin");
                cellData.setCreateTime(new Date());
                cellData.setUpdateBy("admin");
                cellData.setUpdateTime(new Date());
                cellData.setProjectId(projectId);
                cellData.setPeriod(data.getPeriod());
                periodCellDataMapper.insertSelective(cellData);
                data.setCellId(cellData.getId());
            } else {
                PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(data.getCellId());
                if (StringUtils.isNotBlank(data.getKeyinData())) {
                    cellData.setKeyinData(data.getKeyinData());
                    if (StringUtils.isEmpty(cellData.getFormulaExp()))
                        cellData.setFormulaExp(data.getKeyinData());
                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                } else if (data.getAmount() != null && cellData.getData() != data.getAmount().toString()) {
                    //TODO Eddie确认逻辑
//                    cellData.setData(data.getAmount().toString());
//                    if (StringUtils.isEmpty(cellData.getFormulaExp()))
//                        cellData.setFormulaExp(data.getAmount().toString());
//                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                }

            }

            List<DataSourceExtendDto> dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());

            PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
            periodCellTemplateExample.createCriteria().andCellTemplateIdEqualTo(Long.parseLong(data.getCellTemplateId()))
                    .andPeriodEqualTo(data.getPeriod()).andProjectIdEqualTo(projectId);

            Optional<PeriodCellTemplate> periodCellTemplate = periodCellTemplateMapper.selectByExample(periodCellTemplateExample).stream().findFirst();

            BigDecimal originalAmount = new BigDecimal("0");
            PeriodDataSource dataSourceModel = null;
            if (dataSourceExtendDtos.size() > 0) {
                dataSourceModel = dataSourceExtendDtos.get(0).getDataSource();
                if (StringUtils.isBlank(data.getKeyinData()))
                    updateCellValueForDataSourceChange(dataSourceModel, data.getAmount());
                originalAmount = dataSourceModel.getAmount() != null ? dataSourceModel.getAmount() : new BigDecimal("0");
                dataSourceModel.setName(data.getName());
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setAmount(data.getAmount());
                dataSourceModel.setUpdateBy("Admin");
                dataSourceModel.setKeyinData(data.getKeyinData());
                dataSourceModel.setUpdateTime(new Date());
                periodDataSourceMapper.updateByPrimaryKeySelective(dataSourceModel);
            } else {
                dataSourceModel = new PeriodDataSource();
                Long cellDataSourceId = distributedIdService.nextId();
                dataSourceModel.setId(cellDataSourceId);
                dataSourceModel.setType(FormulaDataSourceType.KeyInSource.getCode());
                dataSourceModel.setName(data.getName());
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                dataSourceModel.setAmount(data.getAmount());
                if (periodCellTemplate.isPresent()) {
                    dataSourceModel.setRowIndex(periodCellTemplate.get().getRowIndex());
                    dataSourceModel.setColumnIndex(periodCellTemplate.get().getColumnIndex());
                }
                dataSourceModel.setCreateTime(new Date());
                dataSourceModel.setUpdateTime(new Date());
                dataSourceModel.setCreateBy("admin");
                dataSourceModel.setKeyinData(data.getKeyinData());
                dataSourceModel.setUpdateBy("admin");
                dataSourceModel.setProjectId(projectId);
                dataSourceModel.setPeriod(data.getPeriod());

                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                cellDataSource.setId(distributedIdService.nextId());
                cellDataSource.setCellDataId(data.getCellId());
                cellDataSource.setDataSourceId(cellDataSourceId);
                cellDataSource.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellDataSource.setCreateTime(new Date());
                cellDataSource.setUpdateTime(new Date());
                cellDataSource.setProjectId(projectId);
                cellDataSource.setPeriod(data.getPeriod());
                updateCellValueForDataSourceChange(cellDataSource.getCellDataId(), new BigDecimal("0"), data.getAmount());
                periodDataSourceMapper.insertSelective(dataSourceModel);
                periodCellDataSourceMapper.insertSelective(cellDataSource);
                dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());
            }

            //todo: update the reference cell's data, such as if formula is ND(1)+A2,
            //todo: if A2 is changed manually, ND(1)+A2 result to changed, and datasource of A2 should be updated
            //1. use celltemplateid and period get template id, the change should be in the same sheet.if not in the same sheet, the report should be recalculate, can't changed in time.
            //2. use the celltemplateid get col and row number, get the cell name such as A2,B3 etc.
            String cellName = StringUtils.EMPTY;
            Long templateId = 0L;
            if (periodCellTemplate.isPresent()) {
                Integer colNum = periodCellTemplate.get().getColumnIndex();
                Integer rowNum = periodCellTemplate.get().getRowIndex() + 1;
                cellName = FormulaHelper.excelColIndexToStr(colNum + 1) + rowNum.toString();
                templateId = periodCellTemplate.get().getReportTemplateId();
            }
            //3. use the template id get the reportid, use the reportid get the celltemplateids, then use the celltemplateids and cell name get the datasource which should be updated.
            PeriodReportExample reportExample = new PeriodReportExample();
            reportExample.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(data.getPeriod());
            Optional<PeriodReport> report = periodReportMapper.selectByExample(reportExample).stream().findFirst();
            if (report.isPresent()) {
                Long reportId = report.get().getId();
                PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
                cellDataExample.createCriteria().andReportIdEqualTo(reportId);
                List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(cellDataExample);
                for (PeriodCellData cellData : cellDataList) {
                    if (cellData.getId() != data.getCellId() && cellData.getFormulaExp().contains(cellName)) {
                        PeriodCellDataSourceExample cellDataSourceExample = new PeriodCellDataSourceExample();
                        cellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(cellData.getPeriod()).andCellDataIdEqualTo(cellData.getId()).andCellTemplateIdEqualTo(cellData.getCellTemplateId());
                        List<PeriodCellDataSource> cellDataSourceList = periodCellDataSourceMapper.selectByExample(cellDataSourceExample);
                        for (PeriodCellDataSource cellDataSource : cellDataSourceList) {
                            Long dataSourceId = cellDataSource.getDataSourceId();
                            PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(dataSourceId);
                            if (dataSource.getDescription().equals(cellName) && StringUtils.isBlank(data.getKeyinData())) {
                                BigDecimal changeValue = data.getAmount().subtract(dataSource.getAmount());
                                dataSource.setAmount(data.getAmount());
                                dataSource.setUpdateBy("admin");
                                dataSource.setUpdateTime(new Date());
                                periodDataSourceMapper.updateByPrimaryKeySelective(dataSource);
                                if (!cellData.getData().equals("#VALUE!")) {
                                    if (StringUtils.isBlank(cellData.getData())) cellData.setData("0");
                                    cellData.setData(new BigDecimal(cellData.getData()).add(changeValue).toString());
                                } else {
                                    cellData.setData(new BigDecimal("0.0").add(changeValue).toString());
                                }
                                //cellData.setFormulaExp(cellData.getData());
                                cellData.setUpdateTime(new Date());
                                periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                            } else if (dataSource.getDescription().equals(cellName) && StringUtils.isNotBlank(data.getKeyinData())) {
                                dataSource.setKeyinData(data.getKeyinData());
                                dataSource.setUpdateBy("admin");
                                dataSource.setUpdateTime(new Date());
                                periodDataSourceMapper.updateByPrimaryKeySelective(dataSource);
                            }
                        }
                    }
                }
            }
            //4. before you do the setp3 ,you should get the changed value of datasource. then update the celldata use this change. just add the change value to celldata amount.
            //5. then save the datasource and save the celldata.
            data.setId(dataSourceExtendDtos.get(0).getId());
            operationResultDto.setResult(true);
            operationResultDto.setData(data);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    public OperationResultDto<String> addDataSource(DataSourceDto datasourceDto) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(Long.parseLong(datasourceDto.getCellDataId() != null ? datasourceDto.getCellDataId() : "0"));
            if (cellData == null) {
                operationResultDto.setResult(false);
                operationResultDto.setResultMsg("NoCellData");
                return operationResultDto;
            }

            PeriodDataSource dataSource = new PeriodDataSource();
            CommonUtils.copyProperties(datasourceDto, dataSource);
            dataSource.setType(datasourceDto.getDataSourceType());
            if (dataSource.getId() == null || dataSource.getId() == 0L) {
                dataSource.setId(distributedIdService.nextId());
                dataSource.setCreateTime(new Date());
                dataSource.setCellTemplateId(Long.parseLong(datasourceDto.getCellTemplateId()));
                dataSource.setUpdateTime(new Date());
                dataSource.setUpdateBy(datasourceDto.getUpdater());
                dataSource.setCreateBy(datasourceDto.getCreator());
                dataSource.setPeriod(datasourceDto.getPeriod());
                dataSource.setProjectId(datasourceDto.getProjectId());
            }
            dataSource.setCreateTime(new Date());
            periodDataSourceMapper.insertSelective(dataSource);

            PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
            cellDataSource.setId(distributedIdService.nextId());
            cellDataSource.setCellDataId(cellData.getId());
            cellDataSource.setCellTemplateId(cellData.getCellTemplateId());
            cellDataSource.setOperationType(datasourceDto.getOperationType());
            cellDataSource.setDataSourceId(dataSource.getId());
            cellDataSource.setCreateTime(new Date());
            cellDataSource.setUpdateTime(new Date());
            cellDataSource.setPeriod(dataSource.getPeriod());
            cellDataSource.setProjectId(dataSource.getProjectId());
            periodCellDataSourceMapper.insertSelective(cellDataSource);

            operationResultDto.setData(dataSource.getId().toString());
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    public OperationResultDto<String> getAllDataItems(Integer dataSourceType) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            CellDataSourceType cellDataSourceType = CellDataSourceType.getDSType(dataSourceType);
            String result = StringUtils.EMPTY;
            switch (cellDataSourceType) {
                case OutputInvoice:
                    result = JSON.toJSONString(outputVATInvoiceMapper.selectOutputInvoiceAndItem());
                    break;
                case InputInvoice:
                    result = JSON.toJSONString(inputVATInvoiceMapper.selectInputInvoiceWithItem());
                    break;
                case CustomInvoice:
                    List<CustomsInvoice> customsInvoices = customsInvoiceMapper.selectByExample(null);
                    result = JSON.toJSONString(customsInvoices);
                    break;
                case Formula:
                    break;
                case KeyIn:
                    break;
                case RelatedModel:
                    break;
                case SapDaily:
                    break;
                default:
                    break;
            }
            operationResultDto.setData(result);
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    public OperationResultDto addDataSourceItems(DataSourceDto dataSource) {
        OperationResultDto operationResultDto = new OperationResultDto();

        try {
            if (dataSource.getItems() == null || dataSource.getItems().isEmpty()) {
                operationResultDto.setResultMsg("NoData");
                operationResultDto.setResult(false);
                return operationResultDto;
            }

            if (dataSource == null || StringUtils.isBlank(dataSource.getId())) {
                operationResultDto.setResultMsg("DataSourceInvalid");
                operationResultDto.setResult(false);
                return operationResultDto;
            }

            PeriodDataSource datasourceEntity = periodDataSourceMapper.selectByPrimaryKey(Long.parseLong(dataSource.getId()));
            if (datasourceEntity == null) {
                operationResultDto.setResultMsg("DatasourceInvalid");
                operationResultDto.setResult(false);
                return operationResultDto;
            }
            if (StringUtils.isEmpty(datasourceEntity.getKeyinData()))
                updateCellValueForDataSourceChange(datasourceEntity, dataSource.getAmount());
            periodDataSourceMapper.updateByPrimaryKeySelective(datasourceEntity);
            //todo:MarkProjectModelDirty(dataSource.projectId, dataSource.serviceTypeId, dataSource.Updater);

            List<PeriodDataSourceDetail> dataSourceItems = new ArrayList<>();
            dataSource.getItems().forEach(a -> {
                PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                dataSourceDetail.setId(distributedIdService.nextId());
                dataSourceDetail.setDataSourceId(Long.parseLong(dataSource.getId()));
                dataSourceDetail.setDataSourceType(dataSource.getDataSourceType());
                dataSourceDetail.setItemValue(a);
                dataSourceItems.add(dataSourceDetail);
            });

            PeriodDataSourceDetailExample example = new PeriodDataSourceDetailExample();
            example.createCriteria().andDataSourceIdEqualTo(Long.parseLong(dataSource.getId()));
            List<PeriodDataSourceDetail> dataSourceDetailList = periodDataSourceDetailMapper.selectByExample(example);
            //todo: TBC tomorrow
            dataSourceDetailList.forEach(a -> {
                periodDataSourceDetailMapper.deleteByPrimaryKey(a.getId());
            });

            dataSourceItems.forEach(a -> {
                periodDataSourceDetailMapper.insertSelective(a);
            });

            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    public OperationResultDto<String> getDataSourceDetailList(Long datasSourceId) {
        OperationResultDto<String> operationResultDto = new OperationResultDto<>();
        PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(datasSourceId);
        String itemStrValue = StringUtils.EMPTY;

        if (dataSource.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
            PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
            dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(datasSourceId).andDataSourceTypeEqualTo(CellDataSourceType.InputInvoice.getCode());
            List<PeriodDataSourceDetail> dataSourceDetails = periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
            List<InputVATInvoiceItemExtendDto> inputVATInvoiceItemExtendDtos = new ArrayList<>();
            for (PeriodDataSourceDetail dataSourceDetail : dataSourceDetails) {
                if (StringUtils.isNotBlank(dataSourceDetail.getItemValue())) {
                    InputVATInvoiceItemExtendDto inputVATInvoiceItemExtendDto = inputVATInvoiceMapper.selectInvoiceExtendDto(dataSourceDetail.getItemValue());
                    inputVATInvoiceItemExtendDtos.add(inputVATInvoiceItemExtendDto);
                }
            }
            itemStrValue = JSON.toJSONString(inputVATInvoiceItemExtendDtos);
        } else {
            itemStrValue = StringUtils.EMPTY;
        }

        operationResultDto.setData(itemStrValue);
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    private void updateCellValueForDataSourceChange(Long cellDataId, BigDecimal originalAmount, BigDecimal newAmount) {
        BigDecimal aValue = newAmount != null ? newAmount : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(cellDataId);
        BigDecimal cellVal = new BigDecimal("0");
        try {
            if (cellData != null && StringUtils.isEmpty(cellData.getKeyinData())) {
                cellVal = new BigDecimal(cellData.getData());
                Integer cellDataType = periodCellTemplateMapper
                        .getDataType(cellData.getCellTemplateId()
                                , cellData.getReportId());
                cellData.setData(FormulaHelper.roundValue(cellVal.add(amountChange)
                        , KeyValueConfigResultType.values()[cellDataType], false, null).toString());
            }
        } catch (NumberFormatException ex) {
            logger.error(ex.getMessage(), ex);
        }
    }

    private void updateCellValueForDataSourceChange(PeriodDataSource dataSourceEntity, BigDecimal amount) {
        BigDecimal originalAmount = dataSourceEntity.getAmount();
        dataSourceEntity.setAmount(amount);
        BigDecimal aValue = dataSourceEntity.getAmount() != null ? dataSourceEntity.getAmount() : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        DataSourceCellDataDto dataSourceCellDataDto =
                periodDataSourceMapper.getManualDataSource2(dataSourceEntity.getId());
        BigDecimal cellVal = new BigDecimal("0");
        if (dataSourceCellDataDto != null && dataSourceCellDataDto.getCellData().getData() != null) {
            if (dataSourceCellDataDto.getCellDataSource().getOperationType().equals(EnumOperationType.Sub)) {
                amountChange = BigDecimal.ZERO.subtract(amountChange);
            }

            try {
                cellVal = new BigDecimal(dataSourceCellDataDto.getCellData().getData());
                if (StringUtils.isNotBlank(dataSourceCellDataDto.getCellData().getData())) {
                    Integer cellDataType = periodCellTemplateMapper
                            .getDataType(dataSourceCellDataDto.getCellData().getCellTemplateId()
                                    , dataSourceCellDataDto.getCellData().getReportId());
                    dataSourceCellDataDto.getCellData().setData(
                            FormulaHelper.roundValue(cellVal.add(amountChange),
                                    KeyValueConfigResultType.values()[cellDataType],
                                    false,
                                    null).toString());
                }
            } catch (NumberFormatException ex) {
                logger.error(ex.getMessage(), ex);
            }
        }
    }

    private String getAcctProp(Integer acctProp) {
        switch (acctProp) {
            case 1:
                return "资产";
            case 2:
                return "负债";
            case 3:
                return "共同";
            case 4:
                return "权益";
            case 5:
                return "成本";
            case 6:
                return "损益";
            default:
                return "";
        }
    }

    public Boolean hasManualDataSource(String projectId, Integer period) {
        return periodReportMapper.hasManualDataSource(projectId, period) > 0;
    }

    public String export(String reportData, String serverPath) {
        String filePath = String.format("GeneratedReports\tt_{0}.xlsx", UUID.randomUUID().toString());
        String fullFilePath = combine(serverPath, filePath);
        com.grapecity.documents.excel.Workbook workbook = new com.grapecity.documents.excel.Workbook();
        workbook.fromJson(reportData);
        workbook.save(fullFilePath);
        return filePath;
    }

    private String combine(String parent, String child) {
        return String.format("%s" + File.separator + "%s", parent, child);
    }

    public PeriodJob getRunningJob(String projectId, Integer period) {
        return periodJobMapper.getRunningJob(projectId, period);
    }

    public PeriodJob getJobStatus(String projectId, Integer period, String jobId) {
        PeriodJob job = periodJobMapper.getJobStatus(projectId, period, jobId);
        MyAsserts.assertNotNull(job, Exceptions.NOT_FOUND_EXCEPTION);
        return job;
    }


    @Resource
    private PwcReportAttachMapper pwcReportAttachMapper;

    public PwcReportAttach bindPwcAttach(Long activeCol, Long activeRow, String activeTemplateId, FileUpload file, String remarks) {
        PwcReportAttach pwcReportAttach = new PwcReportAttach();
        pwcReportAttach.setCol(activeCol);
        pwcReportAttach.setCreateTime(file.getCreateTime());
        pwcReportAttach.setRow(activeRow);
        pwcReportAttach.setTemplateId(activeTemplateId);
        pwcReportAttach.setFileName(file.getFileName());
        pwcReportAttach.setUploadUser(file.getCreateBy());
        pwcReportAttach.setFileUrl(file.getViewHttpUrl());
        pwcReportAttach.setFileUploadId(file.getUid());
        pwcReportAttach.setRemarks(remarks);
        pwcReportAttachMapper.insert(pwcReportAttach);
        System.out.println("==>>>附件绑定成功");
        return pwcReportAttach;
    }

    public List<PwcReportAttach> loadAttachList(ReportAttachDto param) {
        PwcReportAttachExample example = new PwcReportAttachExample();
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        PwcReportAttachExample.Criteria criteria = example.createCriteria();
        criteria.andColEqualTo(param.getActiveCol());
        criteria.andRowEqualTo(param.getActiveRow());
        criteria.andTemplateIdEqualTo(param.getActiveTemplateId());
        example.setOrderByClause("create_time DESC");
       /* PageInfo<PwcReportAttach> pageInfo = new PageInfo<PwcReportAttach>();
        pageInfo.setTotal(page.getTotal());*/
        return pwcReportAttachMapper.selectByExample(example);
    }

    /**
     * 删除附件
     *
     * @param id
     * @return
     */
    public OperationResultDto deleteAttach(Long id) {
        PwcReportAttachExample example = new PwcReportAttachExample();
        OperationResultDto<Object> objectOperationResultDto = new OperationResultDto<>();
        example.createCriteria().andIdEqualTo(id);
        int i = pwcReportAttachMapper.deleteByExample(example);
        if (i > 0) {
            objectOperationResultDto.setResultMsg("success");
        }
        return objectOperationResultDto;
    }


    @Transactional
    public void saveDatasource(String orgId, Integer period, Integer specialConsiderations, String ebitRate,
                               OperationResultDto<ReportDataDto> operationResultDto,
                               List<CellDataDto> cellDataList) {
        EbitCellDataExample example = null;

        List<EbitCellData> ebitCellDataList = new ArrayList<>();
        for (int i = 37; i <= 43; i++) {
            example = new EbitCellDataExample();
            EbitCellDataExample.Criteria criteria = example.createCriteria();
            criteria.andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period).andColEqualTo(1).andRowEqualTo(i);
            List<EbitCellData> ebitCellData = ebitCellDataMapper.selectByExample(example);
            if (ebitCellData.size() == 0) {
                for (CellDataDto cellDataDto : cellDataList) {
                    EbitCellData ebitCellData1 = new EbitCellData();
                    ebitCellData1.setId(idService.nextId());
                    ebitCellData1.setOrganizationId(orgId);
                    ebitCellData1.setPeriod(period);
                    ebitCellData1.setCreateTime(new Date());
                    ebitCellData1.setCol(cellDataDto.getColumnIndex());
                    ebitCellData1.setRow(cellDataDto.getRowIndex());
                    ebitCellData1.setData(cellDataDto.getCellValue());
                    ebitCellDataList.add(ebitCellData1);
                }
                ebitCellDataList.add(switchMeth(i, new EbitCellData(), operationResultDto, specialConsiderations, ebitRate, orgId, period));
                ebitCellDataMapper.insertBatch(ebitCellDataList);
            } else {
                ebitCellDataMapper.updateByExampleSelective(switchMeth(i, new EbitCellData(), operationResultDto, specialConsiderations, ebitRate, orgId, period), example);
            }
        }
    }

    private EbitCellData switchMeth(Integer i, EbitCellData ebitCellData1, OperationResultDto<ReportDataDto> operationResultDto, Integer specialConsiderations, String ebitRate, String orgId, Integer period) {
        switch (i) {
            case 37:
                ebitCellData1.setData(operationResultDto.getData().getEbitData().getEbitSubtraction().toString());
                ebitCellData1.setRow(i);
            case 38:
                ebitCellData1.setData(specialConsiderations.toString());
                ebitCellData1.setRow(i);
            case 39:
                ebitCellData1.setData(operationResultDto.getData().getEbitData().getSpecialFactors().toString());
                ebitCellData1.setRow(i);
            case 40:
                ebitCellData1.setData(ebitRate);
            case 41:
                ebitCellData1.setData(operationResultDto.getData().getEbitData().getTransactionAmount().toString());
                ebitCellData1.setRow(i);
            case 42:
                ebitCellData1.setData(operationResultDto.getData().getEbitData().getSixAddTax().toString());
                ebitCellData1.setRow(i);
            case 43:
                ebitCellData1.setData(operationResultDto.getData().getEbitData().getTotalAmountTax().toString());
                ebitCellData1.setRow(i);
        }
        ebitCellData1.setCol(1);
        ebitCellData1.setCreateTime(new Date());
        ebitCellData1.setOrganizationId(orgId);
        ebitCellData1.setPeriod(period);
        return ebitCellData1;
    }


    @Resource
    private EbitSpreadDataMapper ebitSpreadDataMapper;

    @Transactional
    public synchronized OperationResultDto  spreadToDb(HttpServletRequest request, Integer period, String orgId) {
       /* try {
            EbitSpreadDataExample example = new EbitSpreadDataExample();
            EbitSpreadDataExample.Criteria criteria = example.createCriteria();
            EbitSpreadDataExample.Criteria criteria1 = criteria.andOrganizationIdEqualTo(ebitParam.getOrgId()).andPeriodEqualTo(ebitParam.getPeriod());
            List<EbitSpreadData> ebitSpreadData = ebitSpreadDataMapper.selectByExample(example);
            if (ebitSpreadData.size() != 0) {
                EbitSpreadData ebitSpreadData1 = new EbitSpreadData();
                ebitSpreadData1.setSpreadJsonString(ebitParam.getJsonString().getBytes());
                ebitSpreadData1.setOrganizationId(ebitParam.getOrgId());
                ebitSpreadData1.setPeriod(ebitParam.getPeriod());
                ebitSpreadData1.setUpdateTime(new Date());
                ebitSpreadDataMapper.updateByExample(ebitSpreadData1, example);
            } else {
                EbitSpreadData ebitSpreadData1 = new EbitSpreadData();
                ebitSpreadData1.setSpreadJsonString(ebitParam.getJsonString().getBytes());
                ebitSpreadData1.setOrganizationId(ebitParam.getOrgId());
                ebitSpreadData1.setPeriod(ebitParam.getPeriod());
                ebitSpreadData1.setCreateTime(new Date());
                ebitSpreadDataMapper.insert(ebitSpreadData1);
            }
        } catch (Exception e) {
            e.printStackTrace();
            operationResultDto.error();
            operationResultDto.setResultMsg("spread【json序列化字符串】保存失败");
            return operationResultDto;
        }*/
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String,MultipartFile> fileMap = multipartRequest.getFileMap();
        MultipartFile file = fileMap.get("llb.xlsx");
        String fileName = orgId + period + "利润表";
        EbitSpreadDataExample example = new EbitSpreadDataExample();
        example.createCriteria().andPeriodEqualTo(period).andOrganizationIdEqualTo(orgId);
        List<EbitSpreadData> ebitSpreadData1 = ebitSpreadDataMapper.selectByExample(example);
        if(ebitSpreadData1.size() != 0){
            didiFileUploadService.delData(ebitSpreadData1.get(0).getFileKey());
            ebitSpreadDataMapper.deleteByExample(example);
        }

        FileUpload fileUpload = didiFileUploadService.uploadFile(file, fileName, "ebit利润表");

        EbitSpreadData ebitSpreadData = new EbitSpreadData();
        ebitSpreadData.setPeriod(period);
        ebitSpreadData.setOrganizationId(orgId);
        ebitSpreadData.setFileKey(fileUpload.getUid());
        ebitSpreadData.setFileName(fileName);
        ebitSpreadData.setId(idService.nextId());
        ebitSpreadDataMapper.insert(ebitSpreadData);

        return operationResultDto.success();

    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (sheet.getLastRowNum() > 0 &&
                (null == sheet.getRow(0).getCell(0) || "Version".equals(sheet.getRow(0).getCell(0).getStringCellValue())) &&
                null == sheet.getRow(0).getCell(2)) {
            return true;
        }
        return false;
    }

    @Autowired
    TemplateServiceImpl templateService;

    /**
     * 批量导出Excel ebit利润表
     *
     * @param requestParameterDto
     */
    public void manyExport(RequestParameterDto requestParameterDto, String zipFileName, HttpServletRequest request, HttpServletResponse response) throws Exception {
        String zipPath = this.getClass().getResource("/").getPath().substring(1) + "zipDir";
        String fileName = null;
        FileExcelUtil.createFile(zipPath);
        /*  try {
            FileOutputStream out = new FileOutputStream(zipFileName);//要输出的文件名字
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        String filePath;
        File templateFile;
        InputStream inputStream = null;
        List<PeriodTemplate> templates = null;
        templates = templateService.getTemplates(Long.parseLong(requestParameterDto.getTemplateId()));
        MyAsserts.assertNotEmpty(templates, new NotFoundException());
        PeriodTemplate template = templates.get(0);
        String templatePath = template.getPath();
        MyAsserts.assertNotEmpty(templatePath, new NotFoundException());
        filePath = this.getClass().getResource("").toURI().getPath();
        String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
        templateFile = new File(tempPath + templatePath);
        //获取当前期间进行过保存更新的机构的数据
        EbitCellDataExample example = new EbitCellDataExample();
        EbitCellDataExample.Criteria criteria = example.createCriteria();
        criteria.andPeriodEqualTo(requestParameterDto.getPeriod());
        List<EbitCellData> ebitCellDataList = ebitCellDataMapper.selectByExample(example);//获取当前期间下所有的数据
        List<List<EbitCellData>> dataList = Lists.newArrayList();
        List<EbitCellData> orgList = ebitCellDataMapper.selectOrgType(requestParameterDto.getPeriod());
        List<EbitCellData> orgTypeList = Lists.newArrayList();
        File file = null;//要生成的Excel文件
        List<File> filesList = Lists.newArrayList();
        for (int i = 0; i < orgList.size(); i++) {
            orgTypeList = Lists.newArrayList();
            for (EbitCellData ebitCellData : ebitCellDataList) {
                if (!ebitCellData.getOrganizationId().equals(orgList.get(i).getOrganizationId()))
                    continue;
                orgTypeList.add(ebitCellData);

            }
            dataList.add(orgTypeList);
        }
        if (dataList.size() == 0) {
            throw new Exception("没有可导出的数据");
        }
        if (template.getIsSystemType()) {
            try {
                inputStream = new BufferedInputStream(new FileInputStream(templateFile));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        } else {
            if (templatePath.indexOf("/") <= 0) {
                DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                fileParam.setUuids(Arrays.asList(templatePath));
                PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                Map<String, String> urlMap = null;
                if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                    templatePath = uploadDetail.getList().get(0).getViewHttpUrl();
                }
            }
            inputStream = httpFileService.getUserTemplate(templatePath);
            Workbook workbook = WorkbookFactory.create(inputStream);
            Sheet sheet = workbook.getSheetAt(0);
            try {
                Cell cell = null;
                List<Workbook> workbooksList = Lists.newArrayList();
                for (List<EbitCellData> ebitCellDataList1 : dataList) {
                    Workbook newworkbook = new XSSFWorkbook();
                    Sheet sheet1 = newworkbook.getSheetAt(0);
                    if (isSheetEmpty(sheet)) throw new Exception("模板异常");
                    for (EbitCellData ebitCellData : ebitCellDataList1) {
                        loop1:
                        for (int j = 1, k = sheet.getLastRowNum(); j < k; j++) {
                            if (sheet.getRow(j) == null) continue;
                            for (int m = 1, n = sheet.getRow(j).getLastCellNum(); m < n; m++) {
                                cell = sheet.getRow(j).getCell(m);
                                if (cell == null) continue;
                                if (!"".equals(getCellStringValue(cell)) && ebitCellData.getRow() == j && ebitCellData.getCol() == m) {
                                    cell.setCellValue(ebitCellData.getData());
                                    break loop1;
                                }
                            }
                        }
                        sheet1.getRow(ebitCellData.getRow()).getCell(ebitCellData.getCol()).setCellValue(ebitCellData.getData());
                    }
                    POIUtil.cloneSheetAndStyle(workbook.getSheetAt(0), workbook.createSheet("利润表"),newworkbook);
                    workbooksList.add(workbook);
                    //将workbook生成file文件
                    String path = "\\\\zipDir\\\\" + requestParameterDto.getPeriod() + Math.random() + "利润表";
                    file = FileExcelUtil.generateExcelToPath(workbook, path);
                    if (file != null)
                        filesList.add(file);
                }
                FileExcelUtil.downloadZip(request, response, zipFileName, filesList, zipPath);


            } catch (IOException e) {
                e.printStackTrace();
                throw new Exception(ErrorMessage.SystemError);
            }
        }*/
        EbitSpreadDataExample example = new EbitSpreadDataExample();
        example.createCriteria().andPeriodEqualTo(requestParameterDto.getPeriod());
        List<EbitSpreadData> ebitSpreadData = ebitSpreadDataMapper.selectByExample(example);
        List<File> fileList = Lists.newArrayList();
        if (ebitSpreadData.size() == 0)
            throw new Exception("没有可导出的数据");
        for (int i = 0; i < ebitSpreadData.size(); i++) {
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(Arrays.asList(ebitSpreadData.get(i).getFileKey()));
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            String path = null;
            if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                path = uploadDetail.getList().get(0).getViewHttpUrl();
            }
            InputStream inputStream = httpFileService.getUserTemplate(path);
            Workbook workbook1 = WorkbookFactory.create(inputStream);
            fileName =  zipPath + "/" + requestParameterDto.getPeriod() + ".xlsx";
            fileList.add(FileExcelUtil.generateExcelToPath(workbook1, fileName));
        }
        String zipPath1 = zipPath + "/sizegang.zip";
        FileExcelUtil.downloadZip(request, response, zipFileName, fileList, zipPath1);
        //将workbook转成流
               /* ByteArrayOutputStream bos = new ByteArrayOutputStream();
                workbook.write(bos);
                byte[] barray = bos.toByteArray();6
                InputStream is = new ByteArrayInputStream(barray);*/
        // FileOutputStream fileOut = new FileOutputStream(path);
    }

    private String getCellStringValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return cell.getStringCellValue();
        } else if (cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            // 取整
            return String.valueOf((int) cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }
}
